/*
 * Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */


#include "glilo_gui.h"
#include "glilo_sig.h"
#include "glilo_section.h"
#include "glilo_menubar.h"
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <libintl.h>
#include "../lilocommon/memwatch.h"


#define gettext_noop(x) (x)


static GtkItemFactoryEntry menu_items[] = {
  {gettext_noop("/_File"),         NULL,         NULL, 0, "<Branch>"},
  {gettext_noop("/File/_New lilo.conf"),     "<control>N",on_New_lilo_file_activate, 0, NULL},
  {gettext_noop("/File/_Open"),    "<control>O", on_Open_lilo_file_activate, 0, NULL},
/*  {"/File/_Save",    "<control>S", on_Save_lilo_file_activate, 0, NULL},*/
  {gettext_noop("/File/Save _As"), NULL,         on_SaveAs_lilo_file_activate, 0, NULL},
  {gettext_noop("/File/sep1"),     NULL,         NULL, 0, "<Separator>"},
  {gettext_noop("/File/_Quit"),     "<alt>Q", on_Quit_activate, 0, NULL},
  
  {gettext_noop("/_Edit"),         NULL,         NULL, 0, "<Branch>"},
  {gettext_noop("/Edit/_Preview"), "<control>P", on_Preview_activate, 0, NULL},

  {gettext_noop("/_Section"),         NULL,         NULL, 0, "<Branch>"},
  {gettext_noop("/Section/Add _Global"),"<control>G", on_Add_global_activate, 0, NULL},
  {gettext_noop("/Section/Add _Image"), "<control>I", on_Add_image_activate, 0, NULL},
  {gettext_noop("/Section/Add _Other"), "<control>T", on_Add_other_activate, 0, NULL},
  
  {gettext_noop("/_Help"),         NULL,         NULL, 0, "<LastBranch>"},
  {gettext_noop("/_Help/_Report Bugs"),   NULL,         on_Report_bugs_activate, 0, NULL},
  {gettext_noop("/_Help/_About"),   NULL,         on_About_activate, 0, NULL},
};


GtkWidget *get_main_menu (GtkWidget *window)
{
  int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
  int count;
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  /* Modify menu_items to translate menu items */
  for(count=0; count<nmenu_items; count++) {
  menu_items[count].path = g_strdup(gettext(menu_items[count].path));
    };

  /* debugging only, this print out stdout with translated menu items
  for(count=0; count<nmenu_items; count++) {
    printf("menu_items du static : %s\n", menu_items[count]);
    };*/

  accel_group = gtk_accel_group_new();

  /* Initialize the item factory */
  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  /* Lets generate the menus and data about callback functions etc */
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);


  /* Add the accel info */
  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

  /* Now create the menubar and return it */
  return gtk_item_factory_get_widget(item_factory, "<main>");
}



void create_main_window()
{
  GtkWidget *menubar;
  GtkWidget *toolbar;
  
  GtkWidget *vpaned;
  GtkWidget *hpaned;
  
  GtkWidget *treescroll;
  GtkWidget *sectionscroll;
  GtkWidget *keyscroll;
  GtkWidget *textbarscroll;
  
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *vbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox3;

  GtkWidget *vbuttonbox;
  GtkWidget *addbutton;
  GtkWidget *removebutton;
  GtkWidget *updatebutton;

  GtkAccelGroup *accel_group;

  GLILO_GUI *gui; /* The main gui struct */

  gui = malloc(sizeof(GLILO_GUI));

  gui->mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "mainwindow", gui->mainwindow);
  gtk_window_set_title (GTK_WINDOW (gui->mainwindow), "glilo");
  gtk_window_set_policy (GTK_WINDOW (gui->mainwindow), TRUE, TRUE, FALSE);
  gtk_signal_connect(GTK_OBJECT (gui->mainwindow), "delete_event",
                     GTK_SIGNAL_FUNC (on_Quit_activate), NULL);
  gtk_widget_set_usize (gui->mainwindow,540,500);
  
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (gui->mainwindow), vbox1);

  menubar = get_main_menu(gui->mainwindow);
  
  gtk_widget_show (menubar);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, FALSE, 0);


  toolbar = create_toolbar(gui->mainwindow);
  gtk_widget_show (toolbar);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar, FALSE, TRUE, 0);

  
  vpaned = gtk_vpaned_new ();
  gtk_widget_show (vpaned);
  gtk_box_pack_start (GTK_BOX (vbox1), vpaned, TRUE, TRUE, 0);

  hpaned = gtk_hpaned_new ();
  gtk_widget_show (hpaned);
  gtk_container_add (GTK_CONTAINER (vpaned), hpaned);

  treescroll = gtk_scrolled_window_new (NULL,NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(treescroll),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);

  gtk_container_add (GTK_CONTAINER(hpaned),treescroll);
  gtk_widget_show(treescroll);

  gui->lilotree = GTK_CTREE(gtk_ctree_new (1, 0));
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "gui->lilotree", gui->lilotree);
  gtk_widget_show (GTK_WIDGET(gui->lilotree));
  gtk_container_add (GTK_CONTAINER (treescroll), GTK_WIDGET(gui->lilotree));
  gtk_clist_set_selection_mode(GTK_CLIST(gui->lilotree),GTK_SELECTION_BROWSE);
  gtk_ctree_set_line_style (gui->lilotree,GTK_CTREE_LINES_DOTTED);
  gtk_clist_set_column_width (GTK_CLIST (gui->lilotree), 0, 100);
/*  gtk_clist_set_column_width (GTK_CLIST (gui->lilotree), 1, 100);*/
  gtk_clist_column_titles_show (GTK_CLIST (gui->lilotree));
  gtk_signal_connect(GTK_OBJECT(gui->lilotree),"select_row",
                     GTK_SIGNAL_FUNC(on_Lilo_tree_select),gui->lilotree);
  gtk_signal_connect(GTK_OBJECT(gui->lilotree),"tree_collapse",
                     GTK_SIGNAL_FUNC(on_Lilo_tree_collapse),NULL);

  /* We set gui->lilotree as data since we need to block signal in lilo_gtk */

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (hpaned), hbox1);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox3, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox2, TRUE, TRUE, 0);

  sectionscroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (sectionscroll);
  gtk_box_pack_start (GTK_BOX (hbox2), sectionscroll, TRUE, TRUE, 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sectionscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(sectionscroll),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);
  
  gui->sectionlist = gtk_clist_new (1);
  gtk_widget_show (gui->sectionlist);
  gtk_clist_set_selection_mode(GTK_CLIST(gui->sectionlist),GTK_SELECTION_BROWSE);
  gtk_container_add (GTK_CONTAINER (sectionscroll), gui->sectionlist);
  gtk_signal_connect(GTK_OBJECT(gui->sectionlist), "select_row",
                     GTK_SIGNAL_FUNC(on_Section_list_select),gui->sectionlist);

  vbuttonbox = gtk_vbutton_box_new ();
  gtk_widget_show (vbuttonbox);
  gtk_box_pack_start (GTK_BOX (hbox2), vbuttonbox, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (vbuttonbox), 65, 27);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (vbuttonbox), 2, 0);

  addbutton = gtk_button_new_with_label (gettext("Add"));
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "addbutton", addbutton);
  gtk_widget_show (addbutton);
  gtk_container_add (GTK_CONTAINER (vbuttonbox), addbutton);
  gtk_widget_set_usize (addbutton, 26, -1);
  gtk_signal_connect(GTK_OBJECT(addbutton), "clicked",
                     GTK_SIGNAL_FUNC(on_Add_button_click),NULL);

  removebutton = gtk_button_new_with_label (gettext("Remove"));
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "removebutton", removebutton); 
  gtk_widget_show (removebutton);
  gtk_container_add (GTK_CONTAINER (vbuttonbox), removebutton);
  gtk_signal_connect(GTK_OBJECT(removebutton), "clicked",
                     GTK_SIGNAL_FUNC(on_Remove_button_click),NULL);


  /* accelllllls */
  accel_group = gtk_accel_group_new();
  gtk_accel_group_attach(accel_group,GTK_OBJECT(gui->mainwindow));
  gtk_widget_add_accelerator (removebutton,
                              "clicked",
                              accel_group,
                              GDK_F12, /* How does GDK_Delete work ? */
                              0,
                              GTK_ACCEL_VISIBLE | GTK_ACCEL_SIGNAL_VISIBLE);

  gtk_widget_add_accelerator (addbutton,
                              "clicked",
                              accel_group,
                              GDK_Insert,
                              0,
                              GTK_ACCEL_VISIBLE | GTK_ACCEL_SIGNAL_VISIBLE);



  
  /* accelllllls */
  updatebutton = gtk_button_new_with_label (gettext("Update"));
  gtk_object_set_data (GTK_OBJECT (gui->mainwindow), "updatebutton", updatebutton);
  gtk_widget_show (updatebutton);
  gtk_container_add (GTK_CONTAINER (vbuttonbox), updatebutton);
  gtk_signal_connect(GTK_OBJECT(updatebutton), "clicked",
                     GTK_SIGNAL_FUNC(on_Update_button_click),NULL);

  keyscroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (keyscroll);
  gtk_box_pack_start (GTK_BOX (hbox2), keyscroll, TRUE, TRUE, 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (keyscroll),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);

  gui->keylist = gtk_clist_new (1);
  gtk_widget_show (gui->keylist);
  gtk_clist_set_selection_mode(GTK_CLIST(gui->keylist),GTK_SELECTION_BROWSE);
  gtk_container_add (GTK_CONTAINER (keyscroll), gui->keylist);
  gtk_signal_connect(GTK_OBJECT(gui->keylist), "select_row",
                     GTK_SIGNAL_FUNC(on_Keylist_select),gui->keylist);

  
  gui->table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (gui->table1);
  gtk_box_pack_start (GTK_BOX (vbox3), gui->table1, FALSE, TRUE, 10);

  gui->argentry[0] = NULL; /* We will create these two later in lilo_gtk */
  gui->argentry[1] = NULL; /* Depending on the type of widget in use */

  gui->namelabel[0] = gtk_label_new ("");
  gtk_widget_show (gui->namelabel[0]);
  gtk_table_attach (GTK_TABLE (gui->table1), gui->namelabel[0], 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (gui->namelabel[0]), GTK_JUSTIFY_LEFT);

  gui->namelabel[1] = gtk_label_new ("");
  gtk_widget_show (gui->namelabel[1]);
  gtk_table_attach (GTK_TABLE (gui->table1), gui->namelabel[1], 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (gui->namelabel[1]), GTK_JUSTIFY_LEFT);
  gtk_misc_set_padding (GTK_MISC (gui->namelabel[1]), 6, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (vpaned), vbox2);

  textbarscroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (textbarscroll);
  gtk_box_pack_start (GTK_BOX (vbox2), textbarscroll, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (textbarscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  gui->textbar = gtk_text_new (NULL, NULL);
  gtk_widget_show (gui->textbar);
  gtk_container_add (GTK_CONTAINER (textbarscroll), gui->textbar);
  gtk_text_set_editable(GTK_TEXT(gui->textbar),FALSE);
  gtk_text_set_word_wrap(GTK_TEXT(gui->textbar),TRUE);
  GTK_WIDGET_UNSET_FLAGS(gui->textbar,GTK_CAN_FOCUS);

  gui->statusbar = gtk_label_new(gettext("Start the action sonny!..."));
  gtk_box_pack_start (GTK_BOX (vbox2), gui->statusbar, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (gui->statusbar), GTK_JUSTIFY_LEFT);
  gtk_misc_set_padding (GTK_MISC (gui->statusbar), 6, 0);
  gtk_widget_show (gui->statusbar);

  gtk_widget_grab_focus(GTK_WIDGET(gui->lilotree));
  main_gui = gui;
  main_gui->lilodata = NULL;

  on_About_activate (NULL,NULL); /* Put the welcome & (c) text in the textbar */
  hide_illegal_buttons(main_gui); /* We don't wanna show any buttoms during start */
  
  gtk_widget_show(main_gui->mainwindow);
}

