/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <stdio.h>
#include <libintl.h>
#include "glilo_help.h"
#include "glilo_sig.h"
#include "glilo_dialog_input.h"

#include "../lilocommon/message.h"
#include "../lilocommon/lilo_options.h"
#include "../lilocommon/scan.h"
#include "../lilocommon/lilo_help.h"
#include "../lilocommon/memwatch.h"

/*
 *  Do we want the helptext syntaxified ??? (set to '0' otherwise )
 */
#define SYNTAXIFY_WITH_COLOR 1

#define gettext_noop(x) (x)

/*
 * Function to output some helptext to textentry when given a
 * keyword to search for
 */
void show_help (GtkWidget *text,gchar *word)
{
  gchar *helptext=NULL;
  gchar *no_help_found = gettext_noop("Sorry, I didn't find any help for this...");


#if (SYNTAXIFY_WITH_COLOR == 1)
  GdkColormap *cmap;
  GdkColor color[6];
  gchar *curr_str=NULL,*strptr;
  gint i=0,lastcolor=0;
#endif

  helptext = get_help(word,SYNTAXIFY_WITH_COLOR); /* Get help about word, with formatting */

  gtk_text_freeze(GTK_TEXT(text));

  /* Delete the old text */
  gtk_text_backward_delete(GTK_TEXT(text),
                          gtk_text_get_length(GTK_TEXT(text)));

  /* Insert the helptext */
  if (helptext != NULL) {
#if (SYNTAXIFY_WITH_COLOR == 1) /* Format the text */
    /*
     * First we need to allocate some colors...
     */
    cmap = gdk_colormap_get_system();

    /* Normal text */
    color[0].red = 0;
    color[0].green = 0;
    color[0].blue = 0;
    if (!gdk_color_alloc(cmap, &color[0]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* A keyword */
    color[1].red = 0xafff;
    color[1].green = 0;
    color[1].blue = 0;
    if (!gdk_color_alloc(cmap, &color[1]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* A argument to a keyword */
    color[2].red = 0;
    color[2].green = 0;
    color[2].blue = 0xffff;
    if (!gdk_color_alloc(cmap, &color[2]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* anotation */
    color[3].red = 0xfff;
    color[3].green = 0xfff;
    color[3].blue = 0xafff;
    if (!gdk_color_alloc(cmap, &color[3]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* Another keyword, but _not_ the current one */
    color[4].red = 0xfff;
    color[4].green = 0xafff;
    color[4].blue = 0xfff;
    if (!gdk_color_alloc(cmap, &color[4]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* Warning/note */
    color[5].red = 0xffff;
    color[5].green = 0;
    color[5].blue = 0;
    if (!gdk_color_alloc(cmap, &color[5]))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    /* Now lets find what to colorize */

    strptr = helptext;
    while (*strptr != '\0') {
      while (*strptr != '\0' && *strptr != '@')
      {
        strptr++;  /* Move on in the helpword buffer */
        i++;       /* Count characters to format */
      }

      if (i > 0) { /* show string */
        if ((curr_str = calloc (i+1,sizeof(gchar))) == NULL) {
          g_print("Error in %s : %d. Can't calloc memory\n",__FILE__,__LINE__);
          exit(EXIT_FAILURE);
        }
        /* Make a copy of thing to give some colors... */
        strncpy(curr_str,(strptr-i),i);
        gtk_text_insert(GTK_TEXT(text),NULL,&color[lastcolor], NULL,
                   curr_str, strlen(curr_str));

        /* free the temporary copy */
        free (curr_str);
      }

      /* If not eof (null reached), move on past formatting data */
      if (*strptr != '\0') {
        lastcolor = *(strptr+1) - '0';
        strptr+=2;
        i=0;
      }
    }
    
#else /* Okay, no formatting */
    gtk_text_insert(GTK_TEXT(text),NULL,NULL,
                    NULL,helptext,strlen(helptext));
#endif
  /* If no found, insert "Sorry, I didn't ....." */
  }
  else
    gtk_text_insert(GTK_TEXT(text),NULL,NULL,
                    NULL,no_help_found,strlen(gettext(no_help_found)));

  /* Update textedit */
  gtk_text_thaw(GTK_TEXT(text));
  /* Free the helptext pointer */
  if (helptext != NULL)
    free(helptext);
}


void print_message(GtkWidget *textbar,char *message)
{
  gtk_text_freeze(GTK_TEXT(textbar));

  gtk_text_backward_delete(GTK_TEXT(textbar),
                           gtk_text_get_length(GTK_TEXT(textbar)));

  gtk_text_insert(GTK_TEXT(textbar),NULL,NULL,
                  NULL,message,strlen(message));

  gtk_text_thaw(GTK_TEXT(textbar));


}


/*
 * Dumbass
 */
void print_addr(LLIST *list)
{
  int i,max;

  max = llist_get_size(list);
  g_print ("There are %d nodes in it\n",max);
  for (i=0;i<max;i++) {
    g_print ("Node %d == %p\n",i,llist_get_node(list,i));
  }
}



/* eof */
