/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <stdio.h>
#include "glilo_keyword.h"
#include "glilo_section_tree.h"
#include "glilo_sig.h"
#include "glilo_dialog_input.h"

#include "../lilocommon/message.h"
#include "../lilocommon/lilo_keywords.h"
#include "../lilocommon/scan.h"
#include "../lilocommon/lilo_help.h"
#include "../lilocommon/memwatch.h"

#define HIDE_DISABLED_KEYWORDS 0
#define HIDE_USED_KEYWORDS 0


/*
 * Remove and relase all items from the possible keywords list
 */
void remove_data_from_keylist (GtkCList *clist)
{
  LILO_WORD_TO_ADD *data;
  gint i=0;
  
  while ((data = gtk_clist_get_row_data(clist,i++)) != NULL) {
    if (data != NULL) {
      if (data->name != NULL)
        free (data->name);
      free (data);
    }
  }
  gtk_clist_clear(clist);
}


/*
 * Function to recursivly add all keywords to the list
 */
void add_keywords_to_list (GLILO_GUI *gui, LLIST *words,
                           int indent, int parent_row)
{
  GtkCListRow *clist_row;
  gint i;
  gint max;
  gint row_id=0;
  gchar *row;
  POSSIBLE_KEYWORDS *item; /* Holds info about one item (childs,group..) */
  LILO_WORD_TO_ADD *ptr;   /* Set each item in list to this type */

  max = llist_get_size (words);
  for (i=0;i<max;i++) {
    item = llist_get_node (words,i); /* Get current word */

    /* Is the keyword enabled ?, then show it */
    if (item->enabled == KEYWORD_ENABLED ||
        /* Do we wanna show disabled keywords ? */
        (item->enabled == KEYWORD_DISABLED &&
         HIDE_DISABLED_KEYWORDS == 0) ||
        /* Do we wanna show used keywords ? */
        (item->enabled == KEYWORD_USED &&
         HIDE_USED_KEYWORDS == 0)) {

      /* Initialize the structure we use for each keyword in the list */
      ptr = malloc (sizeof(LILO_WORD_TO_ADD));
      ptr->name = calloc(strlen(item->name)+1,sizeof(char));
      strcpy(ptr->name,item->name);
      ptr->group = item->group;

      /* Add space for the text in the keylist */
      row = calloc (strlen(ptr->name)+indent+1,sizeof(gchar));

      sprintf(row,"%*s",indent,""); /* First indent the keyword */
      strcat (row,ptr->name);       /* Then add the keyword */

/*      if (item->enabled == KEYWORD_DISABLED)
        row[0] = '#';

      if (item->enabled == KEYWORD_USED)
        row[0] = '';
*/

      /* Add the row and save the row id */
      row_id = gtk_clist_append(GTK_CLIST(gui->keylist),&row);
      free (row); /* Free the row text */

      /* Disable/Enable the row with a different color (state) */
      if (item->enabled != KEYWORD_ENABLED) {
        clist_row = (g_list_nth (GTK_CLIST(gui->keylist)->row_list, row_id))->data;
        clist_row->state = GTK_STATE_INSENSITIVE;
        /*
         * We don't want the user to select this row
         * I test for GTK_STATE_INSENSITIVE in the
         * handler of the select signal for keylist.
         */
        gtk_clist_set_selectable(GTK_CLIST(gui->keylist),row_id,FALSE);
      }
      /*
       *Set parent_row to actual row if its not a child */
      
      if (parent_row <= 0)
        ptr->parent_row = row_id;
      else
        ptr->parent_row = parent_row; /* Set childs row id to its parent */

      gtk_clist_set_row_data(GTK_CLIST(gui->keylist),row_id,ptr);
    }
    /* Check for childs
     * We keep this one out of the loop
     * since the parent might be disabled
     */
    if (item->childs != NULL) {
      /* If the child is not suppose to be grouped with
       * the parent. Set row_id to 0 to indicate that
       * the child's row won't be associated with the
       * parents row... Look at the end of the loop above
       */
      if (item->group == 0)
        row_id = 0;
      
      add_keywords_to_list (gui,item->childs,indent+3,row_id);
    }
  }
}


/* Loads all possible options to add for the moment to
 * the clist to the very right of the screen
 */
void section_show_possible_keywords(GLILO_GUI *gui)
{
  LLIST *words;

  gtk_clist_freeze(GTK_CLIST(gui->keylist));

  /* Get possible keywords */
  words = get_keywords_for (gui->lilodata,get_section_name(get_last_section_id()),get_last_section());
  /*  print_keywords_for(words); */

  if (words == NULL)
    return;

  /* Add them to the keylist */
  add_keywords_to_list (gui,words,0,-1);

  /* Update the changes */
  gtk_clist_thaw(GTK_CLIST(gui->keylist));


  free_keywords_for(words);

  /* show the add button to might be a good idea */
  gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"addbutton"),TRUE);

  return;
}

