/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <stdio.h>
#include <libintl.h>

#include "glilo_section.h"
#include "glilo_section_tree.h"
#include "glilo_keyword.h"
#include "glilo_help.h"
#include "glilo_sig.h"
#include "glilo_dialog_input.h"

#include "../lilocommon/message.h"
#include "../lilocommon/lilo_options.h"
#include "../lilocommon/scan.h"
#include "../lilocommon/lilo_help.h"
#include "../lilocommon/memwatch.h"

/*
 * This defines how much text can be in the entry widget
 * when the type is 'T'ext or 'P'assword
 */
#define ENTRY_MAX_LENGTH 15

GtkWidget *get_entry_widget (GtkTable *table,GtkWidget *old,LILO_ONE_OPTION *loption,gboolean pos_low);


/*
 * It will always hide the 'update' button. It will hide the
 * 'remove' button unless the parent is the only item left
 */
void hide_illegal_buttons (GLILO_GUI *gui)
{
  GtkCTreeRow *row;

  if (GTK_CLIST(gui->lilotree)->focus_row < 0)
    row = NULL;
  else
    row = GTK_CTREE_ROW (gtk_ctree_node_nth (gui->lilotree,
                                             GTK_CLIST(gui->lilotree)->focus_row));

  /* We wanna check if there are any nodes left to delete or just the parent */
  if ((row != NULL) && (row->parent == NULL &&
                        row->children == NULL && row->sibling == NULL)) {

    /* We wanna show 'remove' if the parent is the only thing left */
    gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"removebutton"),TRUE);
  }
  else
    gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"removebutton"),FALSE);
  /* Hide the 'update' though */
  gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"updatebutton"),FALSE);
  /* and the add button */
  gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"addbutton"),FALSE);

  /* Destroy the widgets where we keep the argument */
  if (gui->argentry[0] != NULL) {
    gtk_widget_destroy(gui->argentry[0]);
    gui->argentry[0] = NULL;
  }
  if (gui->argentry[1] != NULL) {
    gtk_widget_destroy(gui->argentry[1]);
    gui->argentry[1] = NULL;
  }

  /* Set the namelabel for argument to nada */
  gtk_label_set_text(GTK_LABEL(gui->namelabel[0]),"");
  gtk_label_set_text(GTK_LABEL(gui->namelabel[1]),"");
}


/*
 * Loads the hightlighted ctree-section to the 
 * middle key/arg clist.
 */
void load_data_from_section (GLILO_GUI *gui,
                             gint section,gint sec_id)
{
  gint start,end,i,row_counter=0;
  gint next_post_req=-1;
  gchar *row;
  LILO_ONE_OPTION *loption;
  LILO_SECTION_DATA *sec_data;


  gtk_clist_freeze(GTK_CLIST(gui->sectionlist));

  remove_data_from_section(GTK_CLIST(gui->sectionlist));
  remove_data_from_keylist(GTK_CLIST(gui->keylist));

  gtk_clist_clear((GtkCList *) gui->sectionlist);

  set_last_section (section);
  set_last_section_id (sec_id);

  /* Update the keylist to the very right of the gui */
  section_show_possible_keywords(gui);

  if (llist_get_size(gui->lilodata) > 0) {
    /*  g_print("Here...Section=%d, sec_id=%d\n",section,sec_id); */
    start = get_common_node(gui->lilodata,get_section_name(sec_id),section);
    if (sec_id == LILO_GLOBAL) /* image/other/global can all start on node 0 */
      end = start + count_in_global(gui->lilodata);
    else /* Thats why we need separate routines */
      end = start + count_in_common(gui->lilodata,start);
    if (sec_id != LILO_GLOBAL)
      end++;
  }
  else /* We don't want to do the for-loop if list was empty */
    start = end = 0;
  

/*  g_print("There are %d images/others for this section\n",end-start); */

  for (i=start;i<end;i++) {
    loption = llist_get_node(gui->lilodata,i);
    
    sec_data = malloc (sizeof(LILO_SECTION_DATA));

    if (next_post_req >= 0) /* Did the last word req a post req ? */
      sec_data->row = next_post_req; /* Then associate it with prev. row */
    else
      sec_data->row = row_counter; /* else set it to a new row */
    
    if (loption->data_post_req != NULL) {
      sec_data->has_post_arg = row_counter+1; /* we have a post arg here */
      next_post_req = row_counter;
    }
    else {
      sec_data->has_post_arg = -1; /* no req post arg for this word */
      next_post_req = -1;
    }

    /* for clist */
    if (loption->data_args != NULL) {
      row = malloc ((strlen(loption->data_name) +
                          strlen(loption->data_args)+7) * sizeof(gchar));

      if ((strncmp(loption->data_name,"image",5) == 0) ||
          (strncmp(loption->data_name,"other",5) == 0))
        sprintf(row,"%s = %s",loption->data_name,
                loption->data_args);
      else
        sprintf(row,"   %s = %s",loption->data_name,
                loption->data_args);
    }
    else {
      row = malloc ((strlen(loption->data_name)+4) * sizeof(gchar));
      sprintf(row,"   %s",loption->data_name);
    }

    gtk_clist_append ((GtkCList *) gui->sectionlist,&row);
    gtk_clist_set_row_data((GtkCList *) gui->sectionlist,row_counter++,sec_data);
    free (row); /* Free the space since clist_append reservs space anyway */
  }
  gtk_clist_thaw(GTK_CLIST(gui->sectionlist));
}

/*
 * User clicked in the key/arg clist. We need to show
 * the argument and create the widget to use to edit it.
 */
void section_view_part (GtkWidget *clist, GLILO_GUI *gui,
                        LLIST *ldata,gint row)
{
  LILO_ONE_OPTION *loption;
  LILO_SECTION_DATA *sec_data;
  gint node_start;
  gint i,max;
  gchar *str;
  GList *cbitems = NULL;
  GtkBoxChild *child_info=NULL;

  /* First get data for current row */
  sec_data = gtk_clist_get_row_data((GtkCList *) clist,row);

  if (sec_data != NULL)  { /* Has the row some data associated ? */
    /* Then get the data for the row we really want. Usually the same.
     * Only different when clicking on second item for a post-req type.
     * Like when clicking on the "TO" line we want to have "MAP-DRIVE"
     */
    sec_data = gtk_clist_get_row_data((GtkCList *) clist,sec_data->row);

    node_start = get_common_node(ldata,get_section_name(get_last_section_id()),get_last_section());
    node_start += sec_data->row;
    loption = llist_get_node(ldata,node_start); /* get the data */

    /* Lets change the widget type if needed */
    gui->argentry[0] = get_entry_widget (GTK_TABLE(gui->table1),gui->argentry[0],loption,FALSE);

    /* Set the normal entry widget with some data */
    gtk_label_set_text(GTK_LABEL(gui->namelabel[0]),loption->data_name);
    if (loption->data_argtype == ARG_TEXT ||
        loption->data_argtype == ARG_QUOTE ||
        loption->data_argtype == ARG_PASSWD) {
      if (loption->data_args != NULL)
        gtk_entry_set_text(GTK_ENTRY(gui->argentry[0]),loption->data_args);
      else
        gtk_entry_set_text(GTK_ENTRY(gui->argentry[0]),"");

      gtk_widget_set_sensitive(gui->argentry[0],
                               loption->data_args != NULL);
    } /* Set the right text for the entry in the ARG_FILE widget */
    else if (loption->data_argtype == ARG_FILE) {
      child_info = GTK_BOX(gui->argentry[0])->children->data;
      if (loption->data_args != NULL)
        gtk_entry_set_text(GTK_ENTRY(child_info->widget),
                           loption->data_args);
      else
        gtk_entry_set_text(GTK_ENTRY(child_info->widget),"");

    } /* Populate the ARG_COMBO widget with data */
    else if (loption->data_argtype == ARG_COMBO && loption->carg_words != NULL) {
      max = llist_get_size(loption->carg_words);
      for (i=1;i<max;i+=2)
        cbitems = g_list_append(cbitems,
                                llist_get_node(loption->carg_words,i));
      gtk_combo_set_popdown_strings(GTK_COMBO(gui->argentry[0]),
                                              cbitems);
      g_list_free(cbitems);
      /* Now we need to set the right value in the combo */
      str = loption->data_args;

      for (i=0;i<max;i+=2)
        if (g_strcasecmp(str,llist_get_node(loption->carg_words,i)) == 0) {
          str = llist_get_node(loption->carg_words,i+1); /* Found real name */
          i=max;
        }

      gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(gui->argentry[0])->entry),str);

    }
    gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"updatebutton"),
                             loption->data_args != NULL);

    /* show some helptext */
    show_help(gui->textbar,loption->data_name);

    if (sec_data->has_post_arg >= 0) { /* We have a post req too */
      loption = llist_get_node(ldata,++node_start); /* get the next node */
      /* Create the widget for the entry */
      if (loption->data_argtype != ARG_TEXT) {
        g_print("\nI GUESS I'M ABOUT TO SEGFAULT IN %s AT %d\n",__FILE__,__LINE__);
        g_print("THE REASON IS THAT WIDGET FOR 2nd ENTRY CAN\n");
        g_print("JUST BE A GTK-ENTRY FOR THE MOMENT!!!!\n");
        g_print("PLEASE REPORT THIS TO  Per.Winkvist@UK.COM\n\n");
      }
      gui->argentry[1] = get_entry_widget (GTK_TABLE(gui->table1),gui->argentry[1],loption,TRUE);

      gtk_label_set_text(GTK_LABEL(gui->namelabel[1]),loption->data_name);

      if (loption->data_args != NULL)
        gtk_entry_set_text(GTK_ENTRY(gui->argentry[1]),loption->data_args);
      else
        gtk_entry_set_text(GTK_ENTRY(gui->argentry[1]),"");

      /* === NOTE NOTE NOTE NOTE NOTE NOTE ===
       * That the widget for the second option is now limited to
       * beeing a entry!!!!!! A combo box will not work here...
       */
    }
    else {
      if (gui->argentry[1] != NULL) /* Destroy if it not ment to be visible */
        gtk_widget_destroy(gui->argentry[1]);
      gui->argentry[1] = NULL;
      gtk_label_set_text(GTK_LABEL(gui->namelabel[1]),"");
    }
  }
  gtk_widget_set_sensitive(gtk_object_get_data(GTK_OBJECT(gui->mainwindow),"removebutton"),TRUE);
}

/*
 * Return the type of widget to use to edit the keyword argument
 * Currently only Entry and Combobox. Plan to add Entry+Button for files
 */
GtkWidget *get_entry_widget (GtkTable *table,GtkWidget *old,
                             LILO_ONE_OPTION *loption, gboolean pos_low)
{
  gint arg_type;
  GtkWidget *widget=NULL;
  GtkWidget *file_entry=NULL;
  GtkWidget *file_button=NULL;

  arg_type = loption->data_argtype;


  /* ****  POSSIBLE MEMORY LEAK ****
   *
   * Im not sure If I have to do something special to delete
   * the ARG_FILE widget since it has two children (entry+button).
   * Or if the children will die with the parent...
   */
  if (old != NULL)
    gtk_widget_destroy(old);

  if (arg_type == ARG_TEXT || arg_type == ARG_QUOTE
      || arg_type == ARG_PASSWD) {  /* Just make a entry widget */

    widget = gtk_entry_new ();
    /* Set max length for types Text and Password */
    if (arg_type == ARG_TEXT || arg_type == ARG_PASSWD)
      gtk_entry_set_max_length(GTK_ENTRY(widget),ENTRY_MAX_LENGTH);
    gtk_widget_show (widget);
    gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 0+pos_low, 1+pos_low,
                      (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  }
  else if (arg_type == ARG_FILE) { /* Make a entry with a 'file-dialog'-button widget */
    widget = gtk_hbox_new(FALSE,0);
    file_entry = gtk_entry_new ();
    gtk_widget_show(file_entry);
    gtk_box_pack_start (GTK_BOX(widget),file_entry,TRUE,TRUE,0);
    file_button = gtk_button_new_with_label("    . . . .    ");
    gtk_widget_show(file_button);
    gtk_box_pack_start (GTK_BOX(widget),file_button,FALSE,FALSE,0);
    gtk_signal_connect(GTK_OBJECT(file_button), "clicked",
                       GTK_SIGNAL_FUNC(on_GetFile_button_click),file_entry);

    gtk_widget_show (widget);
    gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 0+pos_low, 1+pos_low,
                      (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  }
  else if (arg_type == ARG_COMBO) { /* Make a combo widget */
    widget = gtk_combo_new();
    gtk_widget_show(widget);
    gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 0+pos_low, 1+pos_low,
                      (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  }

  return widget;
}



/*
 * Add a new keyword to the current section
 */
void section_add_option (GLILO_GUI *gui)
{
  LILO_WORD_TO_ADD *ptr;
  LILO_ONE_OPTION *loption;
  gint start_row,row;
  gint size;
  gint section_count;

  size = llist_get_size(gui->lilodata);
  if (size > 0)
    start_row = get_common_node(gui->lilodata,get_section_name(get_last_section_id()),get_last_section());
  else
    start_row = 0;

  if (get_last_section_id() == LILO_GLOBAL)
    if (size > 0) /* is the list != empty ? */
      section_count = count_in_global(gui->lilodata);
    else
      section_count = 0;
  
  else {
    section_count = count_in_common(gui->lilodata,start_row);
    /* We need to add '1' to section_count if current
     * section != LILO_GLOBAL
     */
    section_count ++;
  }
  
  row = ((GtkCList *)gui->keylist)->focus_row;

  /* First get the selected one */
  ptr = gtk_clist_get_row_data(GTK_CLIST(gui->keylist),row);
  /* The get the real one, in case it is a child */
  ptr = gtk_clist_get_row_data(GTK_CLIST(gui->keylist),ptr->parent_row);


  loption = malloc(sizeof(LILO_ONE_OPTION));
  loption->data_name = calloc(strlen(ptr->name)+1,sizeof(char));
  strcpy(loption->data_name,ptr->name);

  /* Add req's and type to the node */
  fill_one_data_with_reqs(loption);

  if (loption->data_argtype != ARG_NADA) { /* Just empty ? */
    loption->data_args = calloc(strlen("<modify this>")+1,sizeof(char));
    strcpy(loption->data_args,"<modify this>");
  }
  else
    loption->data_args = NULL;

  /* Insert the new option to the main list */
  llist_insert_at(gui->lilodata,loption,start_row+section_count);

  if (ptr->group == 1) {
    /* This way of adding the elements should be fixed...
     * It is way to dirty!
     */

    ptr = gtk_clist_get_row_data(GTK_CLIST(gui->keylist),ptr->parent_row+1);
    loption = malloc(sizeof(LILO_ONE_OPTION));
    loption->data_name = calloc(strlen(ptr->name)+1,sizeof(char));
    strcpy(loption->data_name,ptr->name);

    /* Add req's and type to the node */
    fill_one_data_with_reqs(loption);
    if (loption->data_argtype != ARG_NADA) { /* Shall it be empty ? */
      loption->data_args = calloc(strlen("<modify this>")+1,sizeof(char));
      strcpy(loption->data_args,"<modify this>");
    }
    else
      loption->data_args = NULL;

    /* Insert the new option to the main list */
    llist_insert_at(gui->lilodata,loption,start_row+section_count+1);
  }
  /* Finally emit the lilotree signal so everything will be updated */

  gtk_signal_emit_by_name(GTK_OBJECT(gui->lilotree),"select_row");
}

/*
 * Function used by section_remove_option to remove all remaining rows
 * in the keyword/arg clist.
 * It will also remove the entry in the lilotree
 */
void section_remove_all(GLILO_GUI *gui)
{
  LILO_SECTION_DATA *sec_data;
  LILO_ONE_OPTION *loption;
  gint row;

  row = get_common_node(gui->lilodata,get_section_name(get_last_section_id()),get_last_section());

  /* while not more items left in the keyword clist */
  while ((sec_data = gtk_clist_get_row_data((GtkCList *) gui->sectionlist,0)) != NULL) {

    loption = llist_pop_node(gui->lilodata,row);

    gtk_clist_remove (GTK_CLIST(gui->sectionlist),0);

    free (sec_data);
    free_lilo_data_one(loption);
    /* We don't increment row since linked list is adjusted to
     * the right node when we pop the node
     */
  }
  remove_ctree_item(gui->lilotree,gtk_ctree_node_nth(gui->lilotree,
                                                GTK_CLIST(gui->lilotree)->focus_row));
  update_ctree_data(gui->lilotree); /* We must apply section_id changes */
}


/*
 * Function to remove the highligted option in the clist with
 * keywords
 */
void section_remove_option(GLILO_GUI *gui)
{
  LILO_SECTION_DATA *sec_data;
  LILO_ONE_OPTION *loption;
  gint row,start_row;
  gchar *list_text;

  /* We need to block from signals ... */
  gtk_signal_handler_block_by_data (GTK_OBJECT(gui->sectionlist),gui->sectionlist);
  gtk_signal_handler_block_by_data (GTK_OBJECT(gui->lilotree),gui->lilotree);

  if (get_last_section_id() != LILO_GLOBAL)
    start_row = get_common_node(gui->lilodata,get_section_name(get_last_section_id()),get_last_section());
  else
    start_row = 0;
  row = ((GtkCList *)gui->sectionlist)->focus_row;

  if (row == 0) {
    if (gtk_clist_get_row_data((GtkCList *) gui->sectionlist,1) != NULL) {
      gtk_clist_get_text((GtkCList *) gui->sectionlist,0,0,&list_text);
      if (g_strncasecmp(list_text,"image",5) == 0 ||
          g_strncasecmp(list_text,"other",5) == 0) {

        if (list_text[0] == 'i')
          dialog_input_show (gettext("This will remove the entire image. Are you sure ?"),gettext("Confirmation"));
        else
          dialog_input_show (gettext("This will remove the entire \"other\". Are you sure ?"),gettext("Confirmation"));

        if (dialog_was_okay_hit()) /* Did user wanna remove section ? */
          section_remove_all(gui); /* Then remove it... */
        row = -1; /* tell while-loop we're done */
      }
      /* _ELSE_
       * we go straight to the while-loop to just
       * delete one entry
       */
    }
    else {/* We tried to remove row 0 and its the only row left. So remove it! */
      section_remove_all(gui);
      row = -1; /* tell while-loop we're done */
    }
  }

  while (row >=0) { /* We might have a post-req to remove on second trip */
    sec_data = gtk_clist_get_row_data((GtkCList *) gui->sectionlist,row);
    /* Get the one we intend to get instead. In case we selected a
     * post-req we want to remove the parent first, then the post-req
     */
    sec_data = gtk_clist_get_row_data((GtkCList *) gui->sectionlist,sec_data->row);

    loption = llist_pop_node(gui->lilodata,sec_data->row + start_row);

    gtk_clist_remove (GTK_CLIST(gui->sectionlist),sec_data->row);

    if (sec_data->has_post_arg < 0) /* then there is no more to remove */
      row = -1;
    else
      row = sec_data->row; /* Silly :), but since we removed a row, we are at the right one */
    free (sec_data);
    free_lilo_data_one(loption);
  }
  gtk_signal_handler_unblock_by_data (GTK_OBJECT(gui->lilotree),gui->lilotree);
  gtk_signal_handler_unblock_by_data (GTK_OBJECT(gui->sectionlist),gui->sectionlist);

  /* Finally emit the lilotree signal so everything will be updated */
  g_print("Emitting signal...\n");
  gtk_signal_emit_by_name(GTK_OBJECT(gui->lilotree),"select_row");
  g_print("Leaving...\n");
}

/*
 * Function to update an argument for a keyword and to manipulate
 * the linked list with the result
 */
void section_update(GLILO_GUI *gui, LLIST *ldata)
{
  LILO_ONE_OPTION *loption;
  LILO_SECTION_DATA *sec_data;
  gint node_start;
  gchar *str=NULL,*row=NULL;
  gint max,i;
  GtkBoxChild *child_info=NULL;

  /* First get data for current row */
  sec_data = gtk_clist_get_row_data((GtkCList *) gui->sectionlist,((GtkCList *)gui->sectionlist)->focus_row);

  if (sec_data != NULL)  { /* Has the row some data associated ? */
    /* Then get the data for the row we really want. Usually the same.
     * Only different when clicking on second item for a post-req type.
     * Like when clicking on the "TO" line we want to have "MAP-DRIVE"
     */
    sec_data = gtk_clist_get_row_data((GtkCList *) gui->sectionlist,sec_data->row);

    node_start = get_common_node(ldata,get_section_name(get_last_section_id()),get_last_section());
    node_start += sec_data->row;
    loption = llist_get_node(ldata,node_start); /* get the data */

    /* Lets change the widget type if needed */
    if (loption->data_argtype == ARG_TEXT ||
        loption->data_argtype == ARG_QUOTE ||
        loption->data_argtype == ARG_PASSWD) {
      if (loption->data_args != NULL && gui->argentry[0] != NULL)
        str = gtk_entry_get_text(GTK_ENTRY(gui->argentry[0]));

    }
    else if (loption->data_argtype == ARG_FILE) {
      if (loption->data_args != NULL && gui->argentry[0] != NULL) {
        child_info = GTK_BOX(gui->argentry[0])->children->data;
        str = gtk_entry_get_text(GTK_ENTRY(child_info->widget));
      }
    }
    else if (loption->data_argtype == ARG_COMBO  && gui->argentry[0] != NULL) {
      str = gtk_entry_get_text(GTK_ENTRY (GTK_COMBO(gui->argentry[0])->entry));
      max = llist_get_size(loption->carg_words);
      for (i=1;i<max;i+=2) { /* Lets find the real ID for str */
        if (g_strcasecmp(str,llist_get_node(loption->carg_words,i)) == 0) {
          str = llist_get_node(loption->carg_words,i-1); /* Found real ID */
          i=max;
        }
      }
     /* If we don't find real ID, user have probably typed something
      * of his own. Lets use this ID instead then.
      */
    }

    free (loption->data_args);
    loption->data_args = malloc((strlen(str)+1) * sizeof(gchar));

    strcpy(loption->data_args,str);
    row = malloc ((strlen(loption->data_name)+
                   strlen(loption->data_args)+7) * sizeof(gchar));

    if ((strncmp(loption->data_name,"image",5) == 0) ||
          (strncmp(loption->data_name,"other",5) == 0))
        sprintf(row,"%s = %s",loption->data_name,
                loption->data_args);
      else
        sprintf(row,"   %s = %s",loption->data_name,
                loption->data_args);

    gtk_clist_set_text (GTK_CLIST(gui->sectionlist),sec_data->row,0,row);
    free (row);
    
    if (sec_data->has_post_arg >= 0) { /* We have a post req too */
      loption = llist_get_node(ldata,++node_start); /* get the next node */

      str = gtk_entry_get_text(GTK_ENTRY(gui->argentry[1]));

      free (loption->data_args);
      loption->data_args = malloc((strlen(str)+1) * sizeof(gchar));
      strcpy(loption->data_args,str);
      row = malloc ((strlen(loption->data_name)+
                     strlen(loption->data_args)+7) * sizeof(gchar));

      if ((strncmp(loption->data_name,"image",5) == 0) ||
          (strncmp(loption->data_name,"other",5) == 0))
        sprintf(row,"%s = %s",loption->data_name,
                loption->data_args);
      else
        sprintf(row,"   %s = %s",loption->data_name,
                loption->data_args);

      gtk_clist_set_text (GTK_CLIST(gui->sectionlist),sec_data->row+1,0,row);
      free (row);
    }
  }
}

/*
 * Remove and relase all items from the keyword & arg list
 */
void remove_data_from_section (GtkCList *clist)
{
  LILO_SECTION_DATA *data;
  gint i=0;
  
  while ((data = gtk_clist_get_row_data(clist,i++)) != NULL) {
    free (data);
  }
  gtk_clist_clear(clist);
}
