/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <stdio.h>
#include <libintl.h>

#include "glilo_section_tree.h"
#include "glilo_section.h"
#include "glilo_keyword.h"
#include "glilo_help.h"
#include "glilo_sig.h"
#include "glilo_dialog_input.h"

#include "../lilocommon/message.h"
#include "../lilocommon/lilo_options.h"
#include "../lilocommon/scan.h"
#include "../lilocommon/lilo_parse.h"
#include "../lilocommon/lilo_help.h"
#include "../lilocommon/memwatch.h"

#define MAX_TREE_ITEM_LENGTH 100


gint last_section;    /* what section within the section ...*/
gint last_section_id; /* Was it a the global section or an image / other */


/*
 * next_parent is what [+] "section" in the ctree we are at
 * next_child gives us the value of the first child of the parent
 */

int next_parent_value = 0;



/*
 * Create the ctree and fill it with row data. The row data is
 * associated with a LILO_TREE_DATA structure
 */
void fill_ctree_with_data(char *filename, GtkCTree *ctree, LLIST *data)
{
  gchar *text_ptr; /* We just have one column in the tree */
  gchar text_buff[MAX_TREE_ITEM_LENGTH]; /* Yuck, a static buffer */
  LILO_ONE_OPTION *ptr;

  GtkCTreeNode *sibling,*root;
  gint i,max;
  LILO_TREE_DATA *tree_data;

  text_ptr = text_buff;


  gtk_clist_freeze(GTK_CLIST(ctree));

  /*
   * First we have the lilo.conf filename
   */


  root = gtk_ctree_insert_node (ctree,NULL,NULL,&filename,5,
                                NULL,NULL,NULL,NULL,FALSE,FALSE);

  gtk_ctree_expand(ctree,root);
  
  /*
   * Then we add the global section of the file
   * if there is any data in the global section.
   */

  if (count_in_global(data) > 0) {
    strcpy(text_buff,"Global");

    sibling = gtk_ctree_insert_node (ctree,root,NULL,&text_ptr,5,
                                     NULL,NULL,NULL,NULL,FALSE,FALSE);

    tree_data = malloc(sizeof(LILO_TREE_DATA));
    tree_data->parent_id = next_parent_value;
    tree_data->child_id = 0;
    tree_data->section = LILO_GLOBAL;

    gtk_ctree_node_set_row_data(ctree,sibling,tree_data);
  }

  /*
   * Then we grab all kernel images
   */
  
  max = count_images(data);

  for (i=0;i<max;i++) {
    ptr = llist_get_node(data,get_image_node(data,i));
    
    g_snprintf (text_buff,MAX_TREE_ITEM_LENGTH, "Image \"%s\"", ptr->data_args);

    sibling = gtk_ctree_insert_node (ctree,root,NULL,&text_ptr,5,
                                     NULL,NULL,NULL,NULL,FALSE,FALSE);

    tree_data = malloc(sizeof(LILO_TREE_DATA));
    tree_data->parent_id = next_parent_value;
    tree_data->child_id = i;
    tree_data->section = LILO_IMAGE;

    gtk_ctree_node_set_row_data(ctree,sibling,tree_data);
  }

  /*
   * Then we have all other images... MS-DUM and so on...
   */
  
  max = count_others(data);

  for (i=0;i<max;i++) {
    ptr = llist_get_node(data,get_other_node(data,i));

    g_snprintf (text_buff,MAX_TREE_ITEM_LENGTH, "Other \"%s\"", ptr->data_args);

    sibling = gtk_ctree_insert_node (ctree,root,NULL,&text_ptr,5,
                                     NULL,NULL,NULL,NULL,FALSE,FALSE);

    tree_data = malloc(sizeof(LILO_TREE_DATA));
    tree_data->parent_id = next_parent_value;
    tree_data->child_id = i;
    tree_data->section = LILO_OTHER;

    gtk_ctree_node_set_row_data(ctree,sibling,tree_data);
  }

  next_parent_value++; /* Add up id first next lilo file */
  gtk_clist_thaw(GTK_CLIST(ctree));

  /* Im a nice guy since I grab the focus for lilotree here :) */
  gtk_widget_grab_focus(GTK_WIDGET(ctree));
}

/*
 * When removing sections from the lilo tree we have to modify
 * the child_id pointers so that they point to the right section.
 */
void update_ctree_data(GtkCTree *ctree)
{
  gint old_row,i,parent;
  gint last_type=0,count=0;
  GtkCTreeNode *node;
  LILO_TREE_DATA *tree_data;

  gtk_clist_freeze(GTK_CLIST(ctree));
  
  i = old_row = GTK_CLIST(ctree)->focus_row;

  node = gtk_ctree_node_nth (ctree,i);

  if (node == NULL)
    return; /* There are no nodes left!!! */

  /* First we wanna find where the parent is */
  while (GTK_CTREE_ROW(node)->parent != NULL)
    node = gtk_ctree_node_nth (ctree,--i);

  parent = i; /* This is the row for the parent */

  node = gtk_ctree_node_nth (ctree,++i);

  /* Then we wanna find the last child */
  while (node != NULL && GTK_CTREE_ROW(node)->parent != NULL) {
    tree_data = gtk_ctree_node_get_row_data(ctree,node);

    /* Did we change section ? (like global to image) */
    if (tree_data->section != last_type) {
      last_type = tree_data->section;
      count=0; /* Then reset the counter */
    }
    tree_data->child_id = count++;
    gtk_ctree_node_set_row_data(ctree,node,tree_data); /* Apply the changes */

    node = gtk_ctree_node_nth (ctree,++i);
  }

  if (parent == (i-1)) { /* The parent is the only thing left */
    /* Do nothing for the moment */
  }
  
/*  g_print("The end was found at row %d\n",i-1); */
  gtk_ctree_select(ctree,gtk_ctree_node_nth(ctree,old_row));

  gtk_clist_thaw(GTK_CLIST(ctree));
}
  

/*
 * Loads and parses and tries to build up a linked list of the
 * lilo.conf file
 */
LLIST *load_sections_from_file(char *filename,GLILO_GUI *gui)
{
  GdkColormap *cmap;
  GdkColor color;

  LLIST *mess=NULL;
  LLIST *ldata;
  char *str,*str_ptr;
  int lines,i,max;
  

  lines = lilo_parse(filename,&ldata,&mess);

  if (lines > 0) /* success */
  {
    if (ldata != NULL) { /* Did we get anything ? */
      fill_ctree_with_data(filename,gui->lilotree,ldata);
      /* print_data(ldata); */
    }
    g_print(gettext("\nLines read is %d\n"),lines);

  }
  else { /* failure */
    if (ldata != NULL)
      free_lilo_data(ldata);
    clean_up_scan();
    ldata = NULL;
  }
  setmesslist(mess); /* Set message list for printmess() and freemess() */
  
  /*
   * Just output the warnings and errors if
   * there were any...
   */
  if (mess != NULL && llist_get_size(mess) > 0) {
    printmess();

    cmap = gdk_colormap_get_system();    
    /* Set output color to red */
    color.red = 0xafff;
    color.green = 0;
    color.blue = 0;
    if (!gdk_color_alloc(cmap, &color))
      g_error("couldn't allocate color in %s : %d\n",__FILE__,__LINE__);

    gtk_text_freeze(GTK_TEXT(gui->textbar));

    gtk_text_backward_delete(GTK_TEXT(gui->textbar),
                             gtk_text_get_length(GTK_TEXT(gui->textbar)));

    str = calloc (100,sizeof(char));
    sprintf(str,"While loading %s : \n",filename);
    
    gtk_text_insert(GTK_TEXT(gui->textbar),NULL,&color,
                    NULL,str,strlen(str));

    sprintf(str,"Lines : %d\n",lines);
    gtk_text_insert(GTK_TEXT(gui->textbar),NULL,NULL,
                    NULL,str,strlen(str));

    sprintf(str,"Errors/Warnings : %d\n",llist_get_size(mess));
    gtk_text_insert(GTK_TEXT(gui->textbar),NULL,NULL,
                    NULL,str,strlen(str));

    max = llist_get_size(mess);
    if (max > 0) {
      strcpy(str,"\nList of Errors/Warnings :\n");
      gtk_text_insert(GTK_TEXT(gui->textbar),NULL,&color,
                      NULL,str,strlen(str));
      strcpy(str,"\n");
   
      for (i=0;i<max;i++) {
        str_ptr = llist_get_node(mess,i);
        gtk_text_insert(GTK_TEXT(gui->textbar),NULL,NULL,
                        NULL,str_ptr,strlen(str_ptr));

        /* Append a new line */
        gtk_text_insert(GTK_TEXT(gui->textbar),NULL,NULL,
                        NULL,str,strlen(str));

      }
    }
    gtk_text_thaw(GTK_TEXT(gui->textbar));
    free (str);
  }

  freemess(); /* Free the list with messages we got during parse */
  return ldata;
}



/*
 * Adds a new section (either global,image, or other)
 * Note that just one global can exist though.
 * The second argument holds the type to add :
 * LILO_{GLOBAL,IMAGE,OTHER}
 */
void section_add_section (GLILO_GUI *gui, LILO_SECTION sec_type)
{
  GtkCTreeNode *parent_node;
  gchar *lilo_filename = NULL;
  gchar *temp = NULL;
  LLIST *new_node = NULL;
  int node,last_node;
  int i;

  /* Is the tree empty ? */
  if (gtk_ctree_node_nth(gui->lilotree,0) == NULL) {
    gtk_label_set_text(GTK_LABEL(gui->statusbar),
                       gettext("Create a new lilo.conf first! (Ctrl+N)"));
    return;
  }

  /* There are things in the tree ... */
  if (sec_type == LILO_GLOBAL && count_in_global(gui->lilodata) > 0) {
    g_print (gettext("There is already a global section!\n"));
    return;
  }

  if (sec_type == LILO_IMAGE) {
    last_node = count_images(gui->lilodata);

    g_print("imagecount : %d\n",last_node);
    /* Need to check if there are any other images... */
    if (last_node == 0) { /* No previous images */
      if (count_others(gui->lilodata) == 0)
        node = count_in_global(gui->lilodata); /* insert at pos 0 or after last global */
      else
        node = get_other_node(gui->lilodata,0); /* or before first other.. if any */
    }
    else {
      node = get_image_node(gui->lilodata,last_node-1);
      node += count_in_image(gui->lilodata,last_node-1);
      node ++; /* final step */
    }

    g_print("Going to insert at pos : %d\n",node);
    new_node = new_data("image",gettext("<enter path to image>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("label",gettext("<set your label>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("root",gettext("<enter root partition>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("read-only",NULL);
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

  }

  if (sec_type == LILO_OTHER) {
    last_node = count_others(gui->lilodata);

    g_print("othercount : %d\n",last_node);
    /* Need to check if there are any other "others"... */
    if (last_node == 0) { /* No previous  */
      if ((i = count_images(gui->lilodata)) == 0)
        node = count_in_global(gui->lilodata); /* insert at pos 0 or after last global */
      else {
        node = get_image_node(gui->lilodata,i-1); /* or right after last image... */
        node += count_in_image(gui->lilodata,i-1); /* inc node past whole image 'i-1' */
        node ++; /* and a final position */
      }
    }
    else {
      node = get_other_node(gui->lilodata,last_node-1);
      node += count_in_other(gui->lilodata,last_node-1);
      node ++; /* final step */
    }
    
    g_print("Going to insert at pos : %d\n",node);

    new_node = new_data("other",gettext("<enter partition of other>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("label",gettext("<set the label>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("table",gettext("<set table to use>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

  }
  if (sec_type == LILO_GLOBAL) {
    node = 0;

    g_print("Going to insert at pos : %d\n",node);

    new_node = new_data("boot",gettext("<enter boot partition>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("timeout",gettext("<set timeout>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("prompt",NULL);
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("map",gettext("<path to map installer>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);

    new_node = new_data("install",gettext("<enter path to boot.b>"));
    fill_one_data_with_reqs(llist_get_node(new_node,0));  /* Add req's and type to the node */
    llist_insert_at (gui->lilodata,
                     llist_pop_min(new_node),node++);
  }


  /* This could probably be done in a better way.
   * By finding right position and inserting a ctree-node
   * and then reposition the ctree_data pointers....
   */

  /* We need to fetch the name of the parent
   * which is actually the lilo.conf filename
   */

  /* Get selected row */
  i = GTK_CLIST(gui->lilotree)->focus_row;

  /* we need to start from current */
  parent_node = gtk_ctree_node_nth (gui->lilotree,i);
  
  /* Then look backwards for the parent row */
  while (GTK_CTREE_ROW(parent_node)->parent != NULL)
    parent_node = gtk_ctree_node_nth (gui->lilotree,--i);

  /* node now points to parent */
  parent_node = gtk_ctree_node_nth (gui->lilotree,0);
  
  /* get the filename */
  /* Hmmm....
   * I couldn't use gtk_ctree_node_get_text() for parent node
   * and with the lilo_filename. It didn't work.
   * I probably did it wrong, but this works for now.
   */
  
  temp = GTK_CELL_TEXT (GTK_CTREE_ROW (parent_node)->row.cell[0])->text;
  lilo_filename = calloc(strlen(temp)+1,sizeof(gchar));
  strcpy (lilo_filename,temp);

  remove_ctree_data(gui); /* Remove all stuff from the lilodata ctree */

  fill_ctree_with_data(lilo_filename, gui->lilotree,gui->lilodata); /* build it up again */

  free (lilo_filename);
  llist_free (new_node);

/*  print_data(gui->lilodata); */
}


/*
 * Called from clean_up_things in the ctree_post_recursive function
 * to remove & release all the nodes in the ctree for lilo sections
 */
void remove_ctree_item (GtkCTree *ctree, GtkCTreeNode *node)
{
  LILO_TREE_DATA *tree_data;

  tree_data = gtk_ctree_node_get_row_data(ctree,node);
  if (tree_data != NULL) /* We won't free the parent */
    free (tree_data); /* Free data we alloced when creating tree */
  gtk_ctree_remove_node(ctree,node);
}


/*
 * Removes all items from the section list (the very left ctree)
 * It uses remove_items_recursive to clean up the stuff
 */

void remove_ctree_data (GLILO_GUI *gui)
{
  /* expand all, then remove each row and the belonging data */
  gtk_clist_freeze (GTK_CLIST(gui->lilotree));

  gtk_ctree_expand_recursive(gui->lilotree,NULL);

  /* We don't want any signals to be emitted */
  gtk_signal_handler_block_by_data (GTK_OBJECT(gui->lilotree),gui->lilotree);

  /* Remove all data and rows from the lilotree */
  gtk_ctree_post_recursive (gui->lilotree, gtk_ctree_node_nth(gui->lilotree,0),
                            (GtkCTreeFunc) remove_ctree_item, NULL);

  gtk_signal_handler_unblock_by_data (GTK_OBJECT(gui->lilotree),gui->lilotree);
  
  gtk_clist_thaw(GTK_CLIST(gui->lilotree));
  /* then do the update */
}


int get_last_section (void)
{
  return last_section;
}

int get_last_section_id (void)
{
  return last_section_id;
}

void set_last_section (int id)
{
  last_section = id;
}

void set_last_section_id (int id)
{
  last_section_id = id;
}

char *get_section_name(int i)
{
  char *LILO_SECTION_NAME_INT[] = {"global","image","other",NULL};

  return LILO_SECTION_NAME_INT[i];
}

/* eof */
