/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */


#include "lilo_common.h"
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include "llist.h"
#include "memwatch.h"

static LLIST *data_dir = NULL;

/*
 * Add a path to the data_dir LLIST
 * The path sent as arg will have its
 * memory allocated here...
 */
void add_possible_path (char *path)
{
  if (data_dir == NULL)
    data_dir = llist_new();

  llist_insert (data_dir,(char *) strdup(path));
}


/* Removes all paths from memory */
void free_possible_paths ()
{
  int i,max;
  char *data;

  max = llist_get_size(data_dir);

  for (i=0;i<max;i++) {
    data = (char *) llist_pop_min(data_dir);
    free (data);
  }

  llist_free(data_dir);
  data_dir = NULL;
}


/* Checks if a file exists... internal only */
char* check_filepath_exists (char *directory, char *filename)
{
  char *full_filename;
  struct stat s;
  int status;

  full_filename = calloc (strlen (directory) + 1 + strlen (filename) + 1,sizeof(char));
  sprintf(full_filename,"%s/%s",directory,filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;

  free (full_filename);
  return NULL;
}

/*
 * This function will return the path to the file.
 * It won't matter if the file is in the current dir,
 * or if it is somewhere under /usr/share/... or /usr/local/share
 */
char* get_right_filepath(char *filename)
{
  int i=0,max;
  char *file_found = NULL;

  max = llist_get_size(data_dir);

  /* look for the filename in different paths */
  for (i=0;i<max;i++) {
    file_found = check_filepath_exists (llist_get_node(data_dir,i), filename);
    if (file_found)
      break;
  }

  if (!file_found)
  {
    printf ("The file %s wasn't found\n", filename);
    return NULL;
  }
  else
    return file_found;
}

/* eof */
