/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <libintl.h>
#include <string.h>

#include "lilo_help.h"
#include "memwatch.h"
#include "lilo_common.h"

#define BIG_BUFF_SIZE 2000
#define NULL_CHAR '\0'

#define gettext_noop(x) (x)

int strcasecmp (const char *s1, const char *s2);


char *get_help(char *keyword,int syntaxify)
{
  FILE *lilo_file;
  char temp_buff[100];
  char big_temp_buff[BIG_BUFF_SIZE];
  char ch;
  char *helpbuff = NULL;
  int i,max;
  char *lilo_help_file = gettext_noop("lilo_help.dat");

  /* Look for the correct path */
  lilo_help_file = get_right_filepath((char *) gettext(lilo_help_file));

  /* First we must remove spaces infront of the word */
  if (!keyword) {
    printf("The keyword passed to the get_help() function was NULL!");
    return NULL;
  }

  max = strlen(keyword);
  i=0;
  while (i++ < max)
    if (*keyword == ' ')
      keyword++;

  /* NOTE: There can't be any trailing spaces ....*/
  
  if ((lilo_file = fopen(lilo_help_file,"r")) == NULL) {
    perror ("Couldn't open the file! :");
    exit(EXIT_FAILURE);
  }
  free (lilo_help_file); /* Since we called get_right_filepath() we have to free it */
  
  while (!feof(lilo_file)) {

    while (fgetc(lilo_file) != '%' && !feof(lilo_file))
      /* find the keyword marker */ ;

    i = 0;
    while ((ch = fgetc(lilo_file)) != '\n' && ch != ' ' && !feof(lilo_file))
      temp_buff[i++] = ch;

    temp_buff[i] = NULL_CHAR; /* Add end of string */

    /* Compare the string being searched for and current in helpfile */
    if (strcasecmp(temp_buff,keyword) == 0) { /* We found help for the requested keyword */
      i = 0;
      while ((ch = fgetc(lilo_file)) != '%' && !feof(lilo_file)) {
        if (i >= BIG_BUFF_SIZE) {
          printf("Buffer is about to overflow in %s:%d\n",__FILE__,__LINE__);
          printf("i = %d, BIG_BUFF_SIZE = %d\nExiting...\n",i,BIG_BUFF_SIZE);
          exit (EXIT_FAILURE);
        }

        if (ch == '#') { /* A comment was found */
          while (fgetc(lilo_file) != '\n' && !feof(lilo_file))
            /* Move on til end of line */ ;
          big_temp_buff[i++] = '\n';
        }
        else if (ch == '@') {
          if (syntaxify == 0) { /* Remove formatting to helpstring */
            ch = fgetc(lilo_file); /* Get type of formatting */
            if (ch < '0' || ch > '9') { /* something wrong with spec. of formatting */
              printf("I expected a digit (0-9) in the lilo_help.dat file\n");
              printf("Please correct the formatting for keyword %s\n",keyword);
              printf("File : %s Line : %d\n",__FILE__,__LINE__);
              exit(EXIT_FAILURE);
            }
            /* Now we went passed the formatting... */
            
          }
          else { /* Apply formatting to helpstring */
            big_temp_buff[i++] = ch;
            ch = fgetc(lilo_file); /* Apply indication of formatting */
            if (ch < '0' || ch > '9') { /* something wrong with spec. of formatting */
              printf("I expected a digit (0-9) in the lilo_help.dat file\n");
              printf("Please correct the formatting for keyword %s\n",keyword);
              printf("File : %s Line : %d\n",__FILE__,__LINE__);
              exit(EXIT_FAILURE);
            } else
              big_temp_buff[i++] = ch; /* Apply the formatting type*/
          }
        }
        else
          big_temp_buff[i++] = ch;
      }

      /* Now lets remove all trailing spaces, newlines */
      i--; /* go one step back from (from while_loop) */
      while (big_temp_buff[i] == ' ' || big_temp_buff[i] == '\n' ||
             big_temp_buff[i] == '\t')
        i--;
        
      /* Now just apply the NULL character to the string */
      big_temp_buff[++i] = NULL_CHAR;

      /* Now just alloc the help string that we will return */
      if ((helpbuff = calloc (i+1,sizeof(char))) == NULL) {
        printf ("Couldn't callocate memory! %s:%d\n",__FILE__,__LINE__);
        exit (EXIT_FAILURE);
      }
      strncpy(helpbuff,big_temp_buff,i);

      return helpbuff;
    }
  }
  fclose (lilo_file);
  /* Okay we didn't find anything */
  return NULL;
}




  
