/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */


#include <stdlib.h>
#include <string.h>
#include "lilo_options.h"
#include "lilo_data.h"
#include "lilo_common.h"

#include "memwatch.h"

int strcasecmp (const char *s1, const char *s2);

void load_options(LLIST *options, char *filename)
{
  FILE *lilo_file;
  char name_buff[100];
  char count_buff[4];
  char pre_buff[100];
  char post_buff[100];
  char arg_buff[1000];
  char *temp_buff;

  char id_buff[4];
  char op_type;
  LILO_OPTION *lilop;

  /* Get the correct path */
  filename = get_right_filepath(filename);

  /* First we need to open the file */
  if ((lilo_file = fopen(filename,"r")) == NULL) {
    perror ("Couldn't open the file! :");
    exit(EXIT_FAILURE);
  }

  free (filename);  /* Since we called get_right_filepath() we have to free it */
  
  while (!feof(lilo_file)) {
    strcpy(id_buff,""); /* At close to eof it gets dumb otherwise */
    fscanf(lilo_file,"%s",id_buff);
    /* We need to check if id_buff[0] != # (comment) */
    if (IS_CHAR_DIGIT(id_buff[0])) { /* Was it a digit ?*/
      strcpy(pre_buff,"");
      strcpy(post_buff,"");
      fscanf(lilo_file," %c",&op_type); /* Get the option type */
      fscanf(lilo_file," %s",count_buff); /* Get the name */
      fscanf(lilo_file," %s",name_buff); /* Get the name */
      fscanf(lilo_file," %s",pre_buff); /* Get the pre req's. */
      fscanf(lilo_file," %s",post_buff); /* Get the post req's. */
      fgets(arg_buff,sizeof(arg_buff),lilo_file); /* Get whats left as args */

      memmove(pre_buff,pre_buff+4,sizeof(pre_buff)-4); /* Remove "PRE=" */
      memmove(post_buff,post_buff+5,sizeof(post_buff)-5); /* Remove "POST=" */
      memmove(arg_buff,arg_buff+5,sizeof(arg_buff)-5); /* Remove " ARG=" */
      if (arg_buff[strlen(arg_buff)-1] == '\n') /* If we got a newline then */
        arg_buff[strlen(arg_buff)-1] = '\0'; /* Remove the Newline char */

/*      printf("Option ID : %s  (%s) [scope=%c]\n",id_buff,name_buff,op_type);
      printf("pre=[%s] post=[%s] args=[%s]\n\n",pre_buff,post_buff,arg_buff);
  */
   }
    else
      /* If it was a comment, eat up the rest of the line */
      while (fgetc(lilo_file) != '\n' && !feof(lilo_file)); /* Move on until eol */

    /* Now if the stmnt below is true, we have
     * The lilo-command and type of argument.
     */

    if (IS_CHAR_DIGIT(id_buff[0])) {

      /* Create a new lilo option */
      if ((lilop = malloc (sizeof(LILO_OPTION))) == NULL) {
        printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
        exit(EXIT_FAILURE);
      }

      /* Make some space for the name of the option */
      if ((temp_buff = calloc (strlen(name_buff)+1,sizeof(char))) == NULL) {
        printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
        exit(EXIT_FAILURE);
      }
      strcpy (temp_buff,name_buff); /* Make a copy of the read item */
      lilop->op_name = temp_buff;

      
      /*
       * THE POST REQ'S FOR THE LILO KEYWORD
       */
      if (strlen(pre_buff) > 0) {
        /* Make some space for the pre requirements */
        if ((temp_buff = calloc (strlen(pre_buff)+1,sizeof(char))) == NULL) {
          printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
          exit(EXIT_FAILURE);
        }
        strcpy (temp_buff,pre_buff); /* Make a copy of the read item */
        lilop->op_pre = temp_buff;
      }
      else
        lilop->op_pre = NULL; /* No pre's, set it to null */

      /*
       * THE POST REQUIREMENTS FOR THE LILO KEYWORD
       */
      if (strlen (post_buff) > 0) {
        /* Make some space for the post requirements */
        if ((temp_buff = calloc (strlen(post_buff)+1,sizeof(char))) == NULL) {
          printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
          exit(EXIT_FAILURE);
        }
        strcpy (temp_buff,post_buff); /* Make a copy of the read item */
        lilop->op_post = temp_buff;
      }
      else
        lilop->op_post = NULL;

      /*
       * THE ARGUMENTS FOR THE LILO KEYWORD
       */
      if (strlen(arg_buff) > 0) {
        /* Make some space for the arguments */
        if ((temp_buff = calloc (strlen(arg_buff)+1,sizeof(char))) == NULL) {
          printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
          exit(EXIT_FAILURE);
        }
        strcpy (temp_buff,arg_buff); /* Make a copy of the read item */
        lilop->op_args = temp_buff;
      }
      else
        lilop->op_args = NULL;

      /* Set the rest of the things */
      lilop->op_id = atoi(id_buff);
      lilop->op_count = atoi(count_buff);
      lilop->op_type = op_type - '0';

      set_option (options, lilop->op_name,lilop); /* Add the lilop to the linked list */

    }
  }
  fclose (lilo_file);
 /* printf("%d symbols got loaded from the file!\n",llist_get_size(options)); */
}


void free_options(LLIST *options)
{
  LILO_OPTION *ptr;

  while (llist_get_size(options) > 0) {
    ptr = llist_pop_min(options);

    free (ptr->op_name);
    if (ptr->op_pre)
      free (ptr->op_pre);
    if (ptr->op_post)
      free (ptr->op_post);
    if (ptr->op_args)
      free (ptr->op_args);

    free (ptr);
  }
  llist_free (options);
}

LILO_OPTION *get_option_by_id(LLIST *options, int id)
{
  LILO_OPTION *ptr;
  int i,max;

  max = llist_get_size(options);
  for (i=0;i<max;i++) { /* Traverse all options for id */
    ptr = llist_get_node(options,i);
    if (id == ptr->op_id)
      return ptr; /* We found it, so return it */
  }
  return NULL; /* Didn't find */
}

LILO_OPTION *get_option(LLIST *options, char *name)
{
  LILO_OPTION *ptr;
  int i,max;

  max = llist_get_size(options);

  for (i=0;i<max;i++) {
    ptr = llist_get_node (options,i);
    if (strcasecmp(name,ptr->op_name) == 0) /* IGNORE THE CASE */
      return ptr;
  }

/*  printf("The option \"%s\" wasn't found!\n",name); */
  return NULL;
}

/*
 * Returns the integer ID of the option 'name'
 */
int get_option_id (LLIST *options, char *name)
{
  LILO_OPTION *ptr;

  ptr = get_option(options,name);

  if (ptr)
    return ptr->op_id;
  else
    return -1;
}

/* PARAMETER: ARG
 * 0 : Just count the arguments for the word 'name'
 * >0 : Return the argument type for 'name'.
 *   If option type is a ARG_COMBO, then *cargs holds ret values.
 */
 
int get_option_internal(LLIST *options, char *name,
                        int arg, LLIST *cargs)
{
  LILO_OPTION *ptr;
  char *arg_ptr;
  char temp_buff[1000]; /* Used to store values from ARG_COMBO */
  int count_buff;       /* A counter for the temp_buff */
  char *buff;            /* Used for malloced value to llist */
  int arg_count=0;
  int internal_block; /* Used to check for matching pairs in C-block's */

  ptr = get_option(options,name);

  if (ptr == NULL)
    return -1; /* There was no keyword by that name */

  arg_ptr = ptr->op_args;

  while (arg_ptr != NULL && strlen(arg_ptr) > 0) {
    switch (arg_ptr[0]) {
    case 'P' :
      arg_count++;
      if (arg_count == arg && arg > 0)
        return ARG_PASSWD;
      break;

    case 'T' :
      arg_count++;
      if (arg_count == arg && arg > 0)
        return ARG_TEXT;
      break;

    case 'F' :
      arg_count++;
      if (arg_count == arg && arg > 0)
        return ARG_FILE;
      break;
      
    case 'Q'  :
      /* Actually this should just return ARG_Q or arg_count=1 */
      /* But we'll just issue a warning if arg_count exceeds 1 for now */
      arg_count++;
      if (arg_count == arg && arg > 0) /* Do we wanna know the type ? */
        return ARG_QUOTE; /* Then return the type, (see function header) */
      else if(arg_count > 1)
        printf("WARNING: A quoted arg. should be the only arg for %s\n",name);
      break;            /* Since a quoted string can just be by it self */

    case 'C' :
      arg_count++;
      internal_block = 0; /* Used for safety checking of even blocks */
      arg_ptr++; /* Move to the character after 'C' */
      while (arg_ptr != NULL && arg_ptr[0] == ' ')
        arg_ptr++; /* Eat up some space til we bump into the first '"' */
      
      while (arg_ptr!= NULL && arg_ptr[0] == '"') {
        strcpy(temp_buff,"");
        count_buff = -1;
        internal_block++;
        do {
          count_buff++;
          arg_ptr++; /* Move on within the "..." */
          temp_buff[count_buff] = arg_ptr[0];
        } while(arg_ptr != NULL && arg_ptr[0] != '"');

        temp_buff[count_buff] = '\0'; /* Remove the last thing that was added */

        if (arg != 0 && cargs != NULL) { /* Do we wanna return arguments in cargs ? */
          if ((buff = calloc(strlen(temp_buff)+1,sizeof(char))) == NULL) {
            printf("Error: Outof mem? %s : %d\n",__FILE__,__LINE__);
            exit(EXIT_FAILURE);
          }
          strcpy(buff,temp_buff); /* Save the ARG_COMBO word. */
          llist_insert(cargs,buff);
        }

        if (arg_ptr != NULL)
          arg_ptr++; /* Move on to next "..." in the C-block */
        else {
          printf("ERROR: Didn't find matching quote for \"%s\"!\n",name);
          printf("       in get_option_argcount(), check lilo_options.dat\n");
          exit(EXIT_FAILURE);
        }

        while (arg_ptr != NULL && arg_ptr[0] == ' ')
          arg_ptr++; /* Eat up some space til we bump into the next '"' */
      }
      if ((internal_block/2)*2 != internal_block) {
        printf("ERROR: There must be an even amount of\n");
        printf("       blocks in the argument for \"%s\"\n",name);
        exit (EXIT_FAILURE);
      }
      if (internal_block == 0) {
        printf("ERROR: There must be some blocks after a\n");
        printf("       \"c-argument\" for \"%s\"\n",name);
        exit (EXIT_FAILURE);
      }
      arg_ptr--; /* We need to step back once */
      if (arg_count == arg && arg > 0) /* Was the goal to return the type ? */
        return ARG_COMBO; /* Then just return the type */

      break;
    }
    arg_ptr++;
  }
  return arg_count;
}

LILO_OPTION_TYPE get_option_argtype(LLIST *options, char *name, int arg)
{
  int op_type;

  op_type = get_option_internal(options,name,arg,NULL);
  return op_type;
}

int get_option_argcount(LLIST *options, char *name)
{
  int arg_count;

  arg_count = get_option_internal(options,name,0,NULL);
  return arg_count;
}

LLIST *get_option_carg(LLIST *options, char *name, int arg)
{
  int arg_type;
  int blocks;
  LLIST *cargs;

  cargs = llist_new();
  arg_type = get_option_internal(options,name,arg,cargs);

  if (arg_type == ARG_COMBO)
    blocks = llist_get_size(cargs);
  else { /* Some error occured, like no ARG_COMBO was found */
    free_option_carg(cargs); /* Better clean up then */
    return NULL;
  }
  return cargs;
}

void free_option_carg(LLIST *cargs)
{
  char *ptr;

  while (llist_get_size(cargs) > 0) {
    ptr = llist_pop_min(cargs);
    free (ptr);
  }
  llist_free (cargs);
}


void set_option(LLIST *options,char *name,LILO_OPTION *option)
{
  LILO_OPTION *ptr;

  ptr = get_option (options,name);

  if (ptr == NULL) {
/*    printf("Okay, adding it!\n"); */
    llist_insert (options,option); /* Insert the entry to the list */
  }
  else
    printf ("WARNING! The lilo spec file already contained \"%s\".\n",
            ptr->op_name);
}

LILO_OPTION_TYPE get_option_type(LLIST *options, char *name)
{
  LILO_OPTION *ptr;

  ptr = get_option(options,name);
  if (ptr == NULL)
    return -1; /* There was no keyword by that name */
  else
    return ptr->op_type;
}

int is_option_for_scope(LLIST *options,char *name,LILO_SECTION scope)
{
  int option;

  option = get_option_type(options,name);

  if (IS_OPTION_TYPE_FOR_SECTION (option,scope))
    return 1;

  return 0; /* Didn't find a matching scope and option */
}

void print_options(LLIST *options)
{
  int i,max;
  LILO_OPTION *ptr;

  max = llist_get_size(options);
  printf("There are %d options in the list\n\n",max);

  for (i=0;i<max;i++) {
    ptr = llist_get_node(options,i);
    printf("Option ID : %2d (%s)\n",ptr->op_id,ptr->op_name);
/*    printf("Option ID : %d (%s) [args=%d]\n",ptr->op_id,
           ptr->op_name,
           get_option_argcount(options,ptr->op_name));
    printf("pre=[%s] post=[%s] args=[%s]\n\n",ptr->op_pre,
           ptr->op_post,
           ptr->op_args);*/
  }
}

LLIST *get_option_req_internal (LLIST *options, char *name, int which)
{
  LILO_OPTION *ptr; /* pointer to current option */
  LLIST *new_list=NULL; /* The llist we will return with value(s) in it */
  int *list_value;
  int value; /* The current req ID */
  char temp_val[10]; /* Temp value used until ',' is found */
  char *temp_ptr; /* pointer in the temp_val array */
  char *str; /* The pointer to the "which" string */

  ptr = get_option(options,name);
  if (ptr == NULL)
    return NULL; /* There was no keyword by that name */

  new_list = llist_new();
  if (which == 0)      /* We need to set str pointer to right pre/post */
    str = ptr->op_pre;
  else
    str = ptr->op_post;

  /* Okay, we are now ready to look through the string,with,numbers */
  while (str != NULL && strlen(str) > 0) {
    strcpy(temp_val,"");
    temp_ptr = temp_val;
    
    while (str != NULL && strlen(str) > 0 && str[0] != ',') {
      *temp_ptr = str[0];
      temp_ptr++; /* Step on with ptr -> temp_val */
      str++;
    }
    
    if (str[0] == ',') /* Was the current char we stopped at a colon ? */
      str++; /* Then step on. Otherwise, we probably have strlen < 0 */
    *temp_ptr = '\0'; /* Add the null char */

    value = atoi(temp_val);
    if (value > 0) { /* Is it a valid ID ? */
      list_value = malloc(sizeof(int));
      *list_value = value;
      llist_insert(new_list,list_value);
    }
  }
  if (llist_get_size(new_list) < 1) { /* If we didn't get anything */
    llist_free(new_list); /* then deallocate the list memory */
    new_list = NULL;      /* and return null */
  }
  
  return new_list;
}

LLIST *get_option_pre_req (LLIST *options, char *name)
{
  return get_option_req_internal(options,name,0);
}

LLIST *get_option_post_req(LLIST *options,char *name)
{
  return get_option_req_internal(options,name,1);
}

LLIST *get_option_childs(LLIST *options,char *name)
{
  LILO_OPTION *ptr;
  LLIST *new_list=NULL;
  LLIST *pre;
  int *new_int;
  int search_id;
  int i,max;
  int j,jmax;
  int *temp;

  ptr = get_option (options,name);
  search_id = ptr->op_id;

  max = llist_get_size(options);
  new_list = llist_new();
  for (i=0;i<max;i++) {
    ptr = llist_get_node(options,i);
    
    pre = get_option_pre_req(options,ptr->op_name);

    if (pre != NULL) {
      jmax = llist_get_size(pre);
      for (j=0;j<jmax;j++) {
        temp = llist_get_node(pre,j);
        if (*temp == search_id) { /* We found one */
          new_int = malloc (sizeof(int));
          *new_int = ptr->op_id;
          llist_insert(new_list,new_int);
        }
      }
    }
    free_option_pre_req(pre);
  }
  if (llist_get_size(new_list) <= 0) {
    llist_free(new_list);
    new_list = NULL;
  }

  return new_list;
}


void free_option_pre_req(LLIST *options)
{
  int *ptr;

  if (options == NULL)
    return;
  
  while (llist_get_size(options) > 0) {
    ptr = llist_pop_min(options);
    free (ptr);
  }
  llist_free (options);
}

/*
 * Searches the "post_req" list if the word 'name' is available
 */
int check_post_option(LLIST *lilo_ops,LLIST *post_req,char *name)
{
  LILO_OPTION *option_ptr;
  int *id_ptr;
  int i,max;

  max = llist_get_size(post_req);
  if (max == 0)
    return 1; /* Return okay, since there are no post_req's */
  
  option_ptr = get_option(lilo_ops,name);
  if (option_ptr == NULL) { /* There was no keyword by that name */
    printf("WARNING: get_option failed in is_post_option_right\n");
    return 0; /* Return false */
  }

  for (i=0;i<max;i++) {
    id_ptr = llist_get_node(post_req,i);
    if (*id_ptr == option_ptr->op_id) {
      id_ptr = llist_pop_node(post_req,i);
      free (id_ptr); /* Remove node's data from mem */
      return 1;
    }
  }
  return 0; /* Return FALSE, we MUST have a post arg here for 'name' */
}

/*
 * Adds 'name' to the post_req list
 */
void add_post_option(LLIST *lilo_ops,LLIST *post_req,char *name)
{
  int i,max;
  int *new_val,*temp;
  LLIST *ptr;

  ptr = get_option_post_req(lilo_ops,name);
  max = llist_get_size(ptr);
  for (i=0;i<max;i++) {
    new_val = malloc(sizeof(int));
    temp = llist_get_node(ptr,i); /* get the ID to insert */
    *new_val = *temp;
    llist_insert(post_req,new_val); /* insert to post_req list */
  }
  free_option_pre_req(ptr);
}

/*
 * Searches the pre_req list if the word 'name' is available
 */
int check_pre_option(LLIST *lilo_ops,LLIST *pre_req,char *name)
{
  int i,max;
  int i2,max2;
  int *check_val;
  int *curr_val;
  LLIST *ptr;

  ptr = get_option_pre_req(lilo_ops,name);
  max = llist_get_size(ptr);
  max2 = llist_get_size(pre_req);

  if (max == 0) {
    free_option_pre_req(ptr);
    return 1; /* Return TRUE if word don't require anything */
  }
  
  for (i=0;i<max;i++) { /* for each option in 'name' */
    curr_val = llist_get_node(ptr,i);
    for (i2=0;i2<max2;i2++) { /* check with each ID in 'pre_req' */
      check_val = llist_get_node(pre_req,i2);
      if (*check_val == *curr_val) { /* Do they match ? */
        free_option_pre_req(ptr);
        return 1; /* Then return, since requirement was found */
      }
    }
  }
  
  free_option_pre_req(ptr);
  return 0; /* Otherwise, return false. Required word wasn't found */
}

/*
 * Adds 'name' to the pre_req list
 */
void add_pre_option(LLIST *lilo_ops,LLIST *pre_req, char *name)
{
  int *new_val;
  LILO_OPTION *ptr;

  ptr = get_option(lilo_ops,name); /* Get the option struct for name */
  new_val = malloc(sizeof(int));
  *new_val = ptr->op_id;           /* Then insert name's ID */
  llist_insert(pre_req,new_val);
}

/* eof */
