/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */


#ifndef __LILO_OPTIONS_H__
#define __LILO_OPTIONS_H__

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "llist.h"
#include "section.h"
#include "memwatch.h"


#define IS_CHAR_DIGIT(chr) (chr >= '0' && chr <= '9')

#define IS_OPTION_TYPE_ALL(op_type,sec_id) (op_type == ALL_OPTION)
#define IS_OPTION_TYPE_GLOBAL(op_type,sec_id) (op_type == GLOBAL_OPTION && sec_id == LILO_GLOBAL)
#define IS_OPTION_TYPE_IMAGE(op_type,sec_id) (op_type == IMAGE_OPTION && sec_id == LILO_IMAGE)
#define IS_OPTION_TYPE_OTHER(op_type,sec_id) (op_type == OTHER_OPTION && sec_id == LILO_OTHER)
#define IS_OPTION_TYPE_IMHO(op_type,sec_id) (op_type == IMO_OPTION && (sec_id == LILO_IMAGE || sec_id == LILO_OTHER))

#define IS_OPTION_TYPE_FOR_SECTION(op_type,sec_id) (IS_OPTION_TYPE_ALL(op_type,sec_id) || IS_OPTION_TYPE_GLOBAL(op_type,sec_id) || IS_OPTION_TYPE_IMAGE(op_type,sec_id) || IS_OPTION_TYPE_OTHER(op_type,sec_id) || IS_OPTION_TYPE_IMHO(op_type,sec_id))


typedef enum {
/* 0 */  ALL_OPTION,        /* Can be used everywhere */
/* 1 */  GLOBAL_OPTION,     /* Just for "global" */
/* 2 */  IMAGE_OPTION,      /* Just for "image" */
/* 3 */  OTHER_OPTION,      /* Just for "other" */
/* 4 */  IMO_OPTION         /* Just for "image" and "other" */
} LILO_OPTION_TYPE;

typedef enum {
  /* 0 */ ARG_NADA,         /* No argument :) */
  /* 1 */ ARG_TEXT,         /* Argument is plain text (no spaces though) */
  /* 2 */ ARG_QUOTE,        /* Argument is a quoted string */
  /* 3 */ ARG_PASSWD,       /* Argument is a 'T', but used with passwords */
  /* 4 */ ARG_COMBO,        /* Argument holds a pre. defined set of values */
  /* 5 */ ARG_FILE          /* Arg is a file... NOT USED YET!!!! */
} LILO_ARG_TYPE;

typedef struct {
  int op_id;                /* The ID of the option */
  LILO_OPTION_TYPE op_type; /* The type of the lilo-option */
  char *op_name;            /* The name of the option */
  int op_count;             /* Number of times it can be used */
  char *op_pre;             /* What do we pre require ? */
  char *op_post;            /* What do we post require ? */
  char *op_args;            /* List of the args */
} LILO_OPTION;


/* --- FUNCTIONS FOR LILO_OPTIONS --- */
/* ---------------------------------- */

void load_options(LLIST *options, char *filename);
void free_options(LLIST *options);

LILO_OPTION *get_option_by_id(LLIST *options, int id);
LILO_OPTION *get_option(LLIST *options, char *name);
int get_option_id (LLIST *options, char *name);

LILO_OPTION_TYPE get_option_type(LLIST *options, char *name);

/* returns the argument type for argument 'arg' in list *options */
LILO_ARG_TYPE get_option_argtype(LLIST *options, char *name,int arg);
/* returns the amount of arguments for 'name' */
int get_option_argcount(LLIST *options, char *name);

/* returns a list of words (ARG_C type) for 'name's argument 'arg' */
LLIST *get_option_carg(LLIST *options, char *name, int arg);
/* Removes the carg-list from memory */
void free_option_carg(LLIST *cargs);

/* These two routines returns a llist with integers of required ID's */
LLIST *get_option_pre_req(LLIST *options,char *name);
LLIST *get_option_post_req(LLIST *options,char *name);
void free_option_pre_req(LLIST *options);
#define free_option_post_req(arg) free_option_pre_req(arg)

/* It returns a llist with integers of what childs/pre this word has
 * like "image" has alot of childs
 */
LLIST *get_option_childs(LLIST *options,char *name);
#define free_option_childs(arg) free_option_pre_req(arg)

int check_post_option(LLIST *lilo_ops,LLIST *post_req,char *name);
void add_post_option(LLIST *lilo_ops,LLIST *post_req,char *name);
#define free_all_post_options(arg) free_option_pre_req(arg)

int check_pre_option(LLIST *lilo_ops,LLIST *pre_req,char *name);
void add_pre_option(LLIST *lilo_ops,LLIST *pre_req, char *name);
#define free_all_pre_options(arg) free_option_pre_req(arg)

int is_option_missing_pre(LLIST *lilo_ops,LLIST *so_far,char *name);

void set_option(LLIST *options,char *name,LILO_OPTION *option);
int is_option_for_scope(LLIST *options,char *name,LILO_SECTION scope);

#endif

