/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <stdio.h>
#include "message.h"
#include "memwatch.h"
#include "symtab.h"
#include "llist.h"
#include "lilo_options.h"
#include "lilo_data.h"

void init_scan(FILE *scan_file,LLIST *lilo_ops,LLIST *symtab);
void clean_up_scan(void);
void init_parse();
int yyparse(void);

extern int line_counter;   /* In scanner */
extern int parse_errors;   /* In parser */
extern int parse_warnings; /* in parser */
extern LLIST *lilodata;    /* In parser */

int lilo_parse (char *filename,LLIST **lstruct,LLIST **mess)
{
  FILE *lilo_file;
  LLIST *lilo_ops;
  LLIST *symtable;

  if ((lilo_file = fopen(filename,"r")) == NULL) {
    perror("Couldn't open the file! :");
    printf("%s at line %d\n",__FILE__,__LINE__);
    exit(EXIT_FAILURE);
  }

  lilo_ops = llist_new(); /* Init the linked list */
  load_options (lilo_ops,"lilo_options.dat");

  symtable = create_symtab();

  createmess(); /* create a linked list of messages */

  init_scan(lilo_file,lilo_ops,symtable);	/* Init scanner, read from infile */

  init_parse(symtable);       	/* Init parser */

  if (yyparse() != 0) {  	/* yypars() == 0 if ok, otherwise 1 */
    line_counter = 0;
    clean_up_scan();
  }
  else
    fill_data_with_reqs(lilodata); /* We need to fill in pre/post req's */

  
  free_options(lilo_ops);
  free_symtab(symtable);

  *mess = getmesslist();
  *lstruct = lilodata;

  fclose (lilo_file);
  return line_counter; /* Return number of lines parsed */
}
