/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include <string.h>
#include <libintl.h>

#include "lilo_parse.h"
#include "string.h"
#include "stdlib.h"
#include "memwatch.h"
#include "lilo_data.h" /* for print_data,free_lilo_data,count_* */
#include "llist.h"
#include "scan.h"
#include "message.h"
#include "lilo_common.h"

int main(int argc, char **args)
{
  LLIST *mess; /* Here we'll keep all errors/warnings during parse */
  LLIST *ldata; /* This will be the list from the parsed file */
  int lines;
  
  if (argc != 2) { /* Check if a filename was supplied */
    printf("You need to supply a lilo.conf file to parse!\n");
    printf("Like, \"lparse /etc/lilo.conf\" It won't alter anything...promise :)\n");
    exit (EXIT_FAILURE);
  }

  add_possible_path ("/usr/share/glilo");
  add_possible_path ("/usr/local/share/glilo");
  add_possible_path ("./");

  
  lines = lilo_parse(*(args+1),&ldata,&mess); /* start the actual parse */

  if (lines > 0)  /* success */
  {
    printf("\n\n########### EVERYTHING ############\n");
    if (ldata != NULL) { /* Did we get anything ? */
      print_data(ldata);
      printf("\n");
      printf("Images : %d\n",count_images(ldata));
      printf("Others : %d\n",count_others(ldata));
      printf("Globals: %d\n",count_in_global(ldata));
      free_lilo_data(ldata); /* YOU MUST FREE LILO_DATA !!! */
    }
    
    printf(gettext("\nLines read is %d\n"), lines);
  }
  else /* A parse error occured */
  {
    if (ldata != NULL)
      free_lilo_data(ldata); /* YOU MUST FREE LILO_DATA !!! */
    /* yyparse() didn't return 0. Clean up scanner stuff" */
    clean_up_scan();
  }

  free_possible_paths(); /* Remove them from memory */
  setmesslist(mess); /* Need to specify a message list */
  printmess(); /* Print messages in list */
  freemess(); /* And finally free the list with messages*/

  printf("\nBye...\n");
  return EXIT_SUCCESS;
}
/* EOF */

