/* This is the parser that builds puts the tokens together
 * (comming from the scanner (scan.l)) 
 *
 * Author :
 * Per Winkvist (Per.Winkvist@UK.COM) - 1998,1999
 */


%{
#include <stdio.h>
#include <string.h>

#include "parsdef.h" /* includes the struct pars for parsval */
#include "memwatch.h" /* used to detect memleaks */

#include "message.h"  /* just for error/warning messages */
#include "symtab.h"
#include "llist.h"
#include "lilo_data.h"

#define YYERROR_VERBOSE
#define YYDEBUG 1
#define INSERT_YYERRORS 0  /* Do we wanna have yyerrors in the messlist */

/* We use the SAP to get the symbol in the symbol table
 * which are returned from the scanner as a pos-in-sym-tab */
#define SAP(pos) get_symbol_at_pos(symtab,pos)

int snprintf (char *str, size_t n, const char *format, ... );

/* static int yywarn (char *s); not used for the moment */
static int yyerror (char *s);
int yylex (void);
void reset_from_error (void);

int parse_errors,parse_warnings;

LLIST *symtab;	/* Pointer to the symbol table */
LLIST *lilodata; /* Pointer to the lilo data */

extern int line_counter;

%}


/* Semantic value that a token or projection can have */
%union {
  int  val;			/* Integer values from NUMBER */
  char *string;			/* The point to the symtab string */  
  PARS_TYPE parstype; 
}

/* where to start parse */
%start complete

%token <val>    NUM_T		/* Has a value, val */
%token <val> 	LILO_T		/* Has a value, int (index in symtab) */
%token <val> 	ARG_T		/* Argument for keyword */

%token	        ASSIGN_T
%token	      	COMMA_T

%token <val>   	IMAGE_T
%token <val>    OTHER_T

%type <parstype>     complete,globals,global,images,image,other
%type <parstype>     image_body,image_stmnt,other_body,other_stmnt

%type <string>	     arg_body

%%
/* Okay, here is how the lilo structure is built up */

complete:   globals images
	{
         	$$.result = merge_data($1.result, $2.result);
                lilodata = $$.result; /* We need to save result */
                
        };

globals:	/* epsilon (just empty) */
	{
		 $$.result = NULL;
	}
  
	| global globals
	{
		$$.result = merge_data($1.result, $2.result);
        };

global: LILO_T ASSIGN_T arg_body
	{               	
		$$.result = new_data(SAP($1),$3);
        }
        |
        LILO_T
        {
		$$.result = new_data(SAP($1),NULL);
        }
        |LILO_T error arg_body
        {
            printf("Nu blev det tokigt i '==' \n");
            yyerror ("A '=' was expected after the global lilo keyword!");

	    /* ARG_T is in symtab which is freed later on */
            /* free_lilo_data($3.result); Free image_body */
                
            $$.result = NULL;
            reset_from_error();
            yyerrok;	/* Continue anyway */
        };

        

/* Images is either a "image" or an "other"  */
images :  /* epsilon (just empty) */
	{
		 $$.result = NULL;
        }
        | image images
        {
                $$.result = merge_data($1.result,$2.result);
        }
        | other images
        {
                $$.result = merge_data($1.result,$2.result);		
        }
        ;

/* Here is how an image is built up */
image:  IMAGE_T ASSIGN_T ARG_T image_body
	{
		$$.result = merge_data(new_data(SAP($1),SAP($3)),$4.result);
	}
        |IMAGE_T error ARG_T image_body
	{
          	yyerror("An '=' was expected after the image keyword!");

		/* ARG_T is in symtab which is freed later on */
		if ($4.result != NULL)
        	        free_lilo_data($4.result); /* Free image_body */
                
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */
        }
        |IMAGE_T ASSIGN_T error image_body
	{
          	yyerror("A \"imagefile\" wasn't supplied for the image keyword!");

		if ($4.result != NULL) /* If image got a body, free it */
			free_lilo_data($4.result); /* Free image_body data */
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */
        };

/* Here is how the other thingie is built... */
other:  OTHER_T ASSIGN_T ARG_T other_body
	{
		$$.result = merge_data(new_data(SAP($1),SAP($3)),$4.result);
	}
        |OTHER_T error ARG_T other_body
	{
		yyerror("An '=' was expected after the other keyword!");

		/* ARG_T is in symtab which is freed later on */
		if ($4.result != NULL)
	                free_lilo_data($4.result); /* Free image_body */
                
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */		

        }
        |OTHER_T ASSIGN_T error other_body
	{
		yyerror("A \"otherfile\" wasn't supplied for the other keyword!");

		if ($4.result != NULL) /* If otherimage got a body, free it */
			free_lilo_data($4.result); /* Free image_body data */
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */

        };
        

/* This is the statements inside the image */
image_body: /* epsilon */
	{
		 $$.result = NULL;
        }
        | image_stmnt image_body
        {
        	$$.result = merge_data($1.result,$2.result);
        };

image_stmnt: LILO_T ASSIGN_T arg_body
	{
         	$$.result = new_data(SAP($1),$3);
        }
        | LILO_T
        {
		$$.result = new_data(SAP($1),NULL);
        }
        | LILO_T error arg_body
        {
         	yyerror ("A '=' was expected after the lilo keyword!");

		/* ARG_T is in symtab which is freed later on */
                /* free_lilo_data($3.result);  Free image_body */
                
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */		
        };

/* Here comes the statements inside the other-body */
other_body: /* epsilon */
	{
		 $$.result = NULL;
        }
        | other_stmnt other_body
        {
        	$$.result = merge_data($1.result,$2.result);
        };

other_stmnt: LILO_T ASSIGN_T arg_body
	{
         	$$.result = new_data(SAP($1),$3);
        }
        | LILO_T
        {
		$$.result = new_data(SAP($1),NULL);
        }
        | LILO_T error arg_body
        {
         	yyerror ("A '=' was expected after the lilo keyword!");

		/* ARG_T is in symtab which is freed later on */
               /* free_lilo_data($3.result);  Free image_body */
                
		$$.result = NULL;
                reset_from_error();
                yyerrok;	/* Continue anyway */		
        };
        

arg_body: /* Epsilon */
	{
		$$ = NULL;
        }
        | ARG_T arg_body
        {
        	char *space;
                char *arg;
                int index;

                space = calloc(2,sizeof(char));
                strcpy(space," ");

	/* We need to make a copy of $1 since merge_args free's it */
                arg = calloc(strlen(SAP($1))+1,sizeof(char));
                strcpy(arg,SAP($1));

	/* Do the merge. We now lost arg,space,$2 */
		$$ = merge_args(arg,merge_args(space,$2));

	/* Finally add $$ to the symtab. So we don't get a memleak */
	
        	index = insert_symbol(symtab,$$);
                free ($$);
                $$ = get_symbol_at_pos (symtab,index);
        };

%%
/* C-code that isn't declarations */


  
/* setup some things before we start the parsing. */
void init_parse(LLIST *symtable)
{
  parse_errors = 0;
  parse_warnings = 0;
  lilodata = NULL;
  symtab = symtable; /* Reference the symbol table */
}

/* Called by yyparse on error */
int yyerror(char *s)
{
  char temp[101];

  parse_errors++;		/* Count errors */
  snprintf(temp,100,"Error at line %d: %s",line_counter+1,s);

  if (INSERT_YYERRORS == 1)
  	insertmess(temp); /* append the errormessage */
  else
  	printf("%s\n",temp);

  return 0;
}

/* only warn user... */
/*
int yywarn(char *s) 
{
  char temp[101];

  parse_warnings++;	       

  snprintf(temp,100,"Warning at line %d: %s",line_counter+1,s);
  insertmess(temp);

  return 0;
}
*/

/* EOF */







