/* This is the scanner which does the lexical analysis.
 * It sends all tokens it founds to the parser (pars.y)
 *
 * Author :
 * Per Winkvist (Per.Winkvist@UK.COM) - 1998,1999
 */


%{
/* This is C-code that are allowed in the first section */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "memwatch.h"
#include "llist.h"		/* Just used for LLIST *req_list */
#include "message.h"		/* Holds all pars/scan err/warn messages */
#include "symtab.h"		/* Here is where we store scanned symbols */
#include "parsdef.h"            /* defs for struct pars --> parsval */
#include "pars.h"		/* Parser definition from YACC/bison */
#include "lilo_options.h"	/* The list with lilo options */
#include "section.h"         	/* What scope are we in ? */
#include "scan.h"

#define FIRST_LINE_IN_FILE 1	/* Usually 1 or 0 */
#define ARG_IS_STRING 1000 	/* Used in scan.l to parse quoted strings */

int snprintf (char *str, size_t n, const char *format, ... );
int fileno (FILE *stream);

int line_counter;		/* Count lines read */
int scan_errors;		/* specific for scanner. Tokens that are foo and
				   never reach to the parser. Handle them in '.' */

LILO_SECTION scope;		/* Are we in global,image, or other ?*/

int expect_arg;			/* Do we expect an argument ? */
int expect_assign;		/* Do we expect an ASSIGN_T ? */
LLIST *post_req;		/* This list holds ID's of what must appear next */
LLIST *pre_glob;		/* List keywords used in global */
LLIST *pre_curr;		/* List keywords used in current image/other */

/* EXPECT_ARG is here so that it is possible to have "label=backup". Which
 * contains two keywords. But when parsed label, we expect a arg... And thus
 * forces "backup" to be an arg */
 
LLIST *lilo_ops; /* The lilo keyword file to use */

LLIST *symtab;	/* Here is where we store all data */

/* These are defined in pars.y
 * They just add warning and error messages to the linked-
 * list with messages.
 */

%}

/*
 * This is what we know of...
 */

IMAGE	    "image"
OTHER	    "other"

NUMBER      [0-9]+			      
HEXNUMBER   "0x"[0-9A-Fa-f]*

ID          [A-Za-z/\._-][A-Za-z0-9/\,._-]*

/* Old way of specifying text input (limited in characters)*/
/*TEXT	    [A-Za-z0-9/\,\._()&<>\-]*   */

/* New way of specifying text input (glilo - 0.4) */
TEXT    [^" "\n\t\r\"\=\#]*
STRING      [\"].*[\"]

ASSIGN      "="
SINGCOM	    "#"
COMMA	    ","

ENVSTRING1  "${"{ID}"}"
ENVSTRING2  "$"{ID} 



/*
 * %noyywrap here or function yywrap() last.
 */

/* Here starts the real work */
%%

{IMAGE}	{  
		check_expect_arg ();

  		yylval.val = insert_symbol(symtab,yytext);

                scope = LILO_IMAGE;
                expect_arg = 1;
                expect_assign = 1;
                /* Set up some new keyword dependancy */
                free_all_pre_options(pre_curr);
                pre_curr = llist_new();
                add_pre_option(lilo_ops,pre_curr,yytext);
                
		return IMAGE_T;
	}


{OTHER}	{
		check_expect_arg ();

  		yylval.val = insert_symbol(symtab,yytext);

                scope = LILO_OTHER;
                expect_arg = 1;                
                expect_assign = 1;
                /* Set up some new keyword dependancy */
                free_all_pre_options(pre_curr);
                pre_curr = llist_new();
                add_pre_option(lilo_ops,pre_curr,yytext);

		return OTHER_T;
	}

{SINGCOM} {
		while(input() != '\n')
		 /* do nothing */;
}

{ASSIGN}   {
		if (expect_arg == 0)
                	error_expect_keyword();

	     expect_assign = 0;                        
             return ASSIGN_T;
           }

{COMMA}	   {

           	return COMMA_T; /* Used with "Serial" option */
           }


{ENVSTRING1} {  /* ie: ${PATH} */
		char *temp = (char *) strdup (yytext);
                char *ptr;

                temp[strlen(temp)-1] = '\0';
		ptr=temp;
                ptr+=2;
                
		if ((getenv(ptr)) == NULL)
                	error_no_env_found();
		free(temp);
		REJECT; /* Just parse on... We have an error if not found */
	   }

{ENVSTRING2} { /* ie: $PATH */
		char *temp = (char *) strdup (yytext);
                char *ptr;

		ptr = temp;
                ptr++;

		if ((getenv(ptr)) == NULL)
                	error_no_env_found();
		free(temp);
		REJECT; /* Just parse on... We have an error if not found */
	   }

{STRING} { /* A quoted string */
		yylval.val = insert_symbol(symtab,yytext);

		if (expect_arg == ARG_IS_STRING) {
                        expect_arg = 0;
			return ARG_T;
                }
                else 
                	error_no_quote_here();                
                	
	   }


	/* 99-05-16, changed "{ID} | {ENV...." to "{TEXT} | {ENV..."
	 * to allow a digit as first character in unquoted string.  
	 */

 
{TEXT}|{ENVSTRING1}|{ENVSTRING2} { /* An ID with no spaces, must start with char */
  		yylval.val = insert_symbol(symtab,yytext);

		/* printf("I received |%s|\n",yytext); */

		if (expect_assign > 0)  
                	error_expect_assign();
                          	                  
                /* First look if it is a lilo keyword */
		if (expect_arg == 0 && get_option(lilo_ops,yytext) != NULL) {

		      /*printf("Found keyword  : (%s)\n",yytext);*/

			/* If the pre-requirement wasn't found in either
			   global keywords or current image/other then error */
			if (check_pre_option(lilo_ops,pre_glob,yytext) != 1 &&
                            check_pre_option(lilo_ops,pre_curr,yytext) != 1)
                        	error_expect_other_keyword_first();
                
                    	if (check_post_option(lilo_ops,post_req,yytext) != 1)
                        	error_expect_other_keyword();
                    
                	if (is_option_for_scope(lilo_ops,yytext,scope) != 1)
				error_wrong_scope();
                                                                                            
                 	expect_arg = get_option_argcount(lilo_ops,yytext);
                        if (get_option_argtype(lilo_ops,yytext,expect_arg) ==
                        			ARG_QUOTE) 
                        	expect_arg = ARG_IS_STRING; /* quoted string */

                        if (expect_arg > 0 )
                        	expect_assign = 1; /* We need a '=' */

			/* Now we add the post_requirements for this keyword */
			add_post_option(lilo_ops,post_req,yytext);

                        /* We also add it to the list of pre_required words */
                        if (scope == LILO_GLOBAL)
                        	add_pre_option(lilo_ops,pre_glob,yytext);
                        else
                        	add_pre_option(lilo_ops,pre_curr,yytext);
                        return LILO_T; /* Return the lilo keyword */
                }
                else if (expect_arg == 0 && get_option(lilo_ops,yytext) == NULL) {
                 	error_expect_keyword();
                }
                else {
           	    /*    printf("Found argument  : (%s)\n",yytext);  */
                        if (expect_arg == ARG_IS_STRING)
                        	expect_arg = 0;
                        else
	        		expect_arg--; /* One less argument to find */

                	return ARG_T; /* Return keyword argument */
                }
             
           }

{NUMBER}|{HEXNUMBER}  {
		if (expect_arg == 0) /* Try to fix it by not returning a token */
                	error_expect_keyword ();
                else if (expect_assign > 0)
                	error_expect_assign();
                else {
                	if (expect_arg == ARG_IS_STRING)
                        	expect_arg=0;
                        else
	                	expect_arg--;

	             /* Found a number. Save the value for parser */

		     yylval.val = insert_symbol(symtab,yytext);
	         /*  printf("Hittade Num : (%s) (args left = %d)\n",yytext,expect_arg);  */
		     return ARG_T;
                }
           }
	

[\n]       {
	     if (expect_arg == ARG_IS_STRING) {
             	yytext = "new line"; /* Am I stupied now when I modify it ? */
             	error_expect_string(); /* No linebreaks is allowed in a string */
	     }
                
             line_counter++;	/* Count lines */
           }

[\t ]      {
             /* Ignore spaces */
           }

.          { /* It's a point in the beginning of this line */
             /* We need to take care of ALL characters 
              * in the Lexical analyser 
              */

	       char temp[101];

	       scan_errors++;

	       snprintf(temp,100,"Error at line %d: Don't understand \"%s\"",
               			line_counter,yytext);
	       insertmess(temp);

               if (scan_errors > 20) {
               	 strcpy(temp,"The file contained too many illegal chars. Leaving scanner!");
	         insertmess(temp);

		 clean_up_scan();
                 return 1;
               }
           }

%%
/* C-code that should not be at the beginning of resulting code */

/* Initialize code for scanner.
 * Suppose to be called before use of scanner
 */
void init_scan(FILE *infile,LLIST *loops, LLIST *symboltable)
{
  line_counter = FIRST_LINE_IN_FILE;
  lilo_ops = loops;	/* The linked-list with keywords */
  scope = LILO_GLOBAL;	/* What scope we start with */
  expect_arg = 0;	/* We wanna start with a keyword */
  expect_assign = 0;	/* Lets start with something else... */
  scan_errors = 0;

  symtab = symboltable; /* Use this symbol table */
  post_req = llist_new();
  pre_glob = llist_new();
  pre_curr = NULL;
  
  if (infile != NULL)
    yyin = infile;

 /* Here we will delete the current buffer and
  * create a new one. Im not sure if this is done
  * in a proper way... Anyone ???
  *
  * I either have to use one buffer that I yy_delete
  * when done. But then we have to make sure users
  * program really does this (otherwise we get a 16Kb leak)
  * _OR_ We can do like this (what I have done) :
  *  Delete the first/current buffer upon each init/done.
  * Then recreate it the same init == I get no leaks...
  */

  yy_delete_buffer(YY_CURRENT_BUFFER);
  yy_switch_to_buffer (yy_create_buffer(yyin,YY_BUF_SIZE));
}

/* This func will clean up flex's internal buffer */
void clean_up_scan(void)
{
	yy_delete_buffer(YY_CURRENT_BUFFER); /* free the scan buff */
        free_all_post_options(post_req);
        free_all_pre_options(pre_glob);
	free_all_pre_options(pre_curr);
        post_req = NULL;
        pre_glob = NULL;
        pre_curr = NULL;
}


/* Called when Lex found end of file. */
int yywrap()
{
	char temp[101];

	if (expect_arg > 0) {
           snprintf(temp,100,"The last keyword around line %d need to have an argument!\n",line_counter);
           insertmess(temp);
        }

        if (llist_get_size(post_req) > 0) {
           snprintf(temp,100,"A required keyword was left out around line %d!\n",line_counter);
           insertmess(temp);
        }

	clean_up_scan();
  return 1;		 /* We don't want a new file. Stop here */
}

/* Adds a message about keyword is used in wrong scope */
void error_wrong_scope (void)
{
	char temp[101];
        
	snprintf(temp,100,"-- Error around line %d (%s) --",line_counter,yytext);
	insertmess(temp);        
}

/* Check if we expected an argument, then add it to the message list */                       
void check_expect_arg (void)
{
	char temp[101];
        
	if (expect_arg > 0) {
        	snprintf(temp,100,"Argument was expected around line %d. Received \"%s\"",line_counter,yytext);
		insertmess(temp);
        }
}

void error_expect_string (void)
{
	char temp[101];

       	snprintf(temp,100,"A string was expected around line %d. Received \"%s\"",line_counter,yytext);
       	insertmess(temp);

	reset_from_error();
}

/* Check if we expected an keyword, then add it to the message list */
void error_expect_keyword (void)
{
	char temp[101];
        
       	snprintf(temp,100,"Keyword expected around line %d. Received \"%s\"",line_counter,yytext);
       	insertmess(temp);
}

void error_expect_other_keyword_first (void)
{
	char temp[101];
        
       	snprintf(temp,100,"Keyword \"%s\" around line %d requires a missing keyword!",yytext,line_counter);
       	insertmess(temp);
}


void error_expect_other_keyword (void)
{
	char temp[101];
        
       	snprintf(temp,100,"Another keyword was expected around line %d. Received \"%s\"",line_counter,yytext);
       	insertmess(temp);
}


void error_expect_assign (void)
{
	char temp[101];

     	snprintf(temp,100,"An '=' was expected around line %d. Received \"%s\"",line_counter,yytext);
      	insertmess(temp);
        reset_from_error();
}

void error_no_quote_here (void)
{
	char temp[101];

     	snprintf(temp,100,"Keyword don't like quotes around line %d. Received \"%s\"",line_counter,yytext);
      	insertmess(temp);
        reset_from_error();
}

void error_no_env_found (void)
{
       char temp[101];

       snprintf(temp,100,"No such env. variable \"%s\" exists! (around line %d)",yytext,line_counter);
       insertmess(temp);
}

void reset_from_error (void)
{
 	expect_arg = 0;
        expect_assign = 0;
}

/* EOF */
                                                                    