/* Code written by Per Winkvist (Per.Winkvist@UK.COM)
 * In 1999
 * See README and COPYING for further details
 *
 */

#include "symtab.h"
#include "memwatch.h"
#include <string.h>

int strcasecmp (const char *s1, const char *s2);

LLIST *create_symtab (void)
{
  return llist_new ();
}

void free_symtab (LLIST *symtab)
{
  int i,size;
  char *ptr;

  size = llist_get_size (symtab);
  
  for (i=0;i<size;i++) {
    ptr = llist_pop_min (symtab);
    free (ptr); /* Free the symbol */
  }
  llist_free(symtab); /* And finally free the list */
}

int insert_symbol (LLIST *symtab, char *text)
{
  char *data;

  if ((data = calloc (strlen(text)+1,sizeof(char))) == NULL) {
    printf("Couln't allocate memory in %s at %d!\n",__FILE__,__LINE__);
    exit(EXIT_FAILURE);
  }
  strcpy (data,text);

  return llist_insert(symtab,data); /* Return the position */
}

void INTERNAL_print_symtab (char *data)
{
  printf("%s [%x]\n",data,(unsigned int) data);
}

void print_symtab (LLIST *symtab)
{
  printf("symtab size = %d\n",llist_get_size(symtab));
  llist_for_each(symtab,(void *) INTERNAL_print_symtab);
}


/* Get symbol at postion pos.
 * Returns null if it wasn't found */
char *get_symbol_at_pos (LLIST *symtab, int pos)
{
  if (llist_get_size(symtab) <= pos)
    return NULL;

  return (char *) llist_get_node (symtab,pos);
}


/* Does the symbol "text" exist in the list ?
 * If it does, then return its position
 * Else return -1
 * is_case == is the comparison case sensitive ? */
int find_symbol(LLIST *symtab, char *symb,int is_case)
{
  int size;
  int i;
  char *data;

  size = llist_get_size(symtab);

  for (i=0;i<size;i++) {
    data = llist_get_node(symtab,i);
    
    if (is_case == 0 && (strcmp(symb,data) == 0))
      return i;

    if (is_case == 1 && (strcasecmp(symb,data) == 0)) /* IGNORE THE CASE */
      return i;
  }
  return -1; /* Okay, give up! We didn't find it */
}



